package system

import (
	"f5"
	"fmt"
	"main/constant"
	"main/model/system"
	"net/http"
	"q5"
	"strconv"

	"github.com/gin-gonic/gin"
)

type AnncApi struct {
}

type anncJson struct {
	AnncType int32  `json:"annc_type" binding:"required" `
	PlatForm int32  `json:"platform" binding:"required" `
	AnncDesc string `json:"annc_desc" binding:"required" `
}

var anncCfg []anncJson

func (aa *AnncApi) Init() {
	loginanncs := []anncJson{}

	if jsonStr, err := f5.ReadJsonFile("../config/login_annc.json"); err == nil {
		if err := q5.DecodeJson(jsonStr, &loginanncs); err != nil {
			panic(fmt.Sprintf("parse metafile error %s %s", "login_annc.json", err))
		}
	} else {
		panic(fmt.Sprintf("load metafile error %s %s", "login_annc.json", err))
	}

	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	for _, item := range loginanncs {
		if item.AnncType != constant.LOGIN_ANNC_TYPE_LEAST_VERSION &&
			item.AnncType != constant.LOGIN_ANNC_TYPE_WHITE_VERSION &&
			item.AnncType != constant.LOGIN_ANNC_TYPE_CURRENT_VERSION {
			panic(fmt.Sprintf("annc type 配置错误:%d, %d", item.AnncType, item.PlatForm))
		}

		if item.PlatForm != constant.PLATFORM_ANDROID &&
			item.PlatForm != constant.PLATFORM_IOS {
			panic(fmt.Sprintf("annc platform 配置错误:%d, %d", item.AnncType, item.PlatForm))
		}

		annc := new(system.Annc)
		var count int64 = 0
		f5.GetApp().GetOrmDb(constant.CONF_DB).Table(annc.TableName()).Where("annc_type = ?", item.AnncType).Where("platform = ?", item.PlatForm).Count(&count)
		if count > 0 {
			continue
		}

		annc.AnncType = item.AnncType
		annc.Version = "1.0.0"
		annc.ClickType = 0
		annc.PlatForm = item.PlatForm
		annc.CreateTime = nowDaySeconds
		annc.ModifyTime = nowDaySeconds
		if err := f5.GetApp().GetOrmDb(constant.CONF_DB).Create(annc).Error; err != nil {
			f5.GetSysLog().Error("添加annc 失败:%d, %d", item.AnncType, item.PlatForm)
		}
	}

	anncCfg = loginanncs
}

func (this *AnncApi) AnncList(c *gin.Context) {
	var anncList []*struct {
		system.Annc
		Desc string `json:"desc"`
	}

	err := f5.GetApp().GetOrmDb(constant.CONF_DB).Find(&anncList).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	for _, item := range anncList {
		for _, cfg := range anncCfg {
			if item.AnncType == cfg.AnncType && item.PlatForm == cfg.PlatForm {
				item.Desc = cfg.AnncDesc
				break
			}
		}
	}

	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
		"data":    anncList,
	})
}

func (this *AnncApi) AddAnnc(c *gin.Context) {
	annc := system.Annc{}
	if err := c.ShouldBindJSON(&annc); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	if !this.CheckLoginAnnc(annc, c) {
		return
	}

	var count int64
	db := f5.GetApp().GetOrmDb(constant.CONF_DB)
	db.Table(annc.TableName()).Where("annc_type = ?", annc.AnncType).Where("platform = ?", annc.PlatForm).Count(&count)
	if count > 0 {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "无法再插入同类型的公告",
		})
		return
	}

	err := db.Create(&annc).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
	})
}

func (this *AnncApi) UpdateAnnc(c *gin.Context) {
	idx, _ := strconv.ParseUint(c.Param("idx"), 10, 64)
	var count int64
	annc := system.Annc{}
	if err := c.ShouldBindJSON(&annc); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	if !this.CheckLoginAnnc(annc, c) {
		return
	}

	db := f5.GetApp().GetOrmDb(constant.CONF_DB)
	db.Table(annc.TableName()).Where("idx = ?", idx).Count(&count)
	if count < 1 {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "不存在的数据",
		})
		return
	}

	// db.Table(annc.TableName()).Where("idx != ?", idx).Where("annc_type = ?", annc.AnncType).Where("platform = ?", annc.PlatForm).Count(&count)
	// if count > 0 {
	// 	c.JSON(http.StatusOK, gin.H{
	// 		"code":    1,
	// 		"message": "已有同类型的公告",
	// 	})
	// 	return
	// }

	annc.ModifyTime = int32(f5.GetApp().GetRealSeconds())
	err := db.Select("*").Omit("idx", "createtime", "annc_type", "platform").Where("idx = ?", idx).Updates(&annc).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
	})
}

func (this *AnncApi) IngameAnncList(c *gin.Context) {
	anncList := []system.IngameAnnc{}
	err := f5.GetApp().GetOrmDb(constant.CONF_DB).Find(&anncList).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
		"data":    anncList,
	})
}

func (this *AnncApi) AddIngameAnnc(c *gin.Context) {
	annc := system.IngameAnnc{}
	if err := c.ShouldBindJSON(&annc); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	if !this.CheckIngameAnnc(annc, c) {
		return
	}

	annc.Uniid = uint64(f5.GetApp().NewNodeUuid())
	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	annc.BeginTime = int32(int64(annc.BeginTime) - q5.GetDaySeconds(int64(annc.BeginTime), 0))
	annc.EndTime = int32(int64(annc.EndTime) - q5.GetDaySeconds(int64(annc.EndTime), 0))
	annc.BeginDate = int32(q5.GetDaySeconds(int64(annc.BeginDate), 0))
	annc.EndDate = int32(q5.GetDaySeconds(int64(annc.EndDate), 0))
	annc.CreateTime = nowDaySeconds
	annc.ModifyTime = nowDaySeconds
	err := f5.GetApp().GetOrmDb(constant.CONF_DB).Create(&annc).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
	})
}

func (this *AnncApi) UpdateIngameAnnc(c *gin.Context) {
	idx, _ := strconv.ParseUint(c.Param("idx"), 10, 64)
	annc := system.IngameAnnc{}
	if err := c.ShouldBindJSON(&annc); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	if !this.CheckIngameAnnc(annc, c) {
		return
	}

	var count int64
	db := f5.GetApp().GetOrmDb(constant.CONF_DB)
	db.Table(annc.TableName()).Where("idx = ?", idx).Count(&count)
	if count < 1 {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "不存在的公告",
		})
		return
	}

	annc.BeginTime = int32(int64(annc.BeginTime) - q5.GetDaySeconds(int64(annc.BeginTime), 0))
	annc.EndTime = int32(int64(annc.EndTime) - q5.GetDaySeconds(int64(annc.EndTime), 0))
	annc.BeginDate = int32(q5.GetDaySeconds(int64(annc.BeginDate), 0))
	annc.EndDate = int32(q5.GetDaySeconds(int64(annc.EndDate), 0))
	annc.ModifyTime = int32(f5.GetApp().GetRealSeconds())
	err := db.Select("*").Omit("idx", "uniid", "createtime").Where("idx = ?", idx).Updates(&annc).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
	})
}

func (this *AnncApi) CheckLoginAnnc(annc system.Annc, c *gin.Context) bool {
	if len(annc.Title) > 0xFF {
		c.JSON(http.StatusOK, gin.H{
			"code":    2,
			"message": "标题过长",
		})
		return false
	}

	if len(annc.Content) > 0xFF {
		c.JSON(http.StatusOK, gin.H{
			"code":    2,
			"message": "内容过长",
		})
		return false
	}

	return true
}

func (this *AnncApi) CheckIngameAnnc(annc system.IngameAnnc, c *gin.Context) bool {
	if len(annc.Title) > 0xFF {
		c.JSON(http.StatusOK, gin.H{
			"code":    2,
			"message": "标题过长",
		})
		return false
	}

	if len(annc.Content) > 0xFF {
		c.JSON(http.StatusOK, gin.H{
			"code":    2,
			"message": "内容过长",
		})
		return false
	}

	notsecs := f5.GetApp().GetRealSeconds()
	// if annc.EndTime < annc.BeginTime || annc.EndTime < int32(notsecs) {
	// 	c.JSON(http.StatusOK, gin.H{
	// 		"code":    2,
	// 		"message": "发送时间不对",
	// 	})
	// 	return false
	// }

	if q5.GetDaySeconds(int64(annc.EndDate), 0) < q5.GetDaySeconds(int64(annc.BeginDate), 0) ||
		q5.GetDaySeconds(int64(annc.EndDate), 0) < q5.GetDaySeconds(int64(notsecs), 0) {
		c.JSON(http.StatusOK, gin.H{
			"code":    2,
			"message": "发送日期不对",
		})
		return false
	}

	return true
}
