package system

import (
	"f5"
	"fmt"
	"main/constant"
	"main/model/system"
	"net/http"
	"q5"
	"strconv"

	"github.com/gin-gonic/gin"
)

type AuditApi struct {
}

func (aa *AuditApi) Init() {
	audits := []struct {
		Version string `json:"version" binding:"required" `
		Model   uint   `json:"model" binding:"required" `
	}{}

	if jsonStr, err := f5.ReadJsonFile("../config/audit.json"); err == nil {
		if err := q5.DecodeJson(jsonStr, &audits); err != nil {
			panic(fmt.Sprintf("parse metafile error %s %s", "audit.json", err))
		}
	} else {
		panic(fmt.Sprintf("load metafile error %s %s", "audit.json", err))
	}

	for _, item := range audits {
		audit := new(system.Audit)
		var count int64 = 0
		f5.GetApp().GetOrmDb(constant.CONF_DB).Table(audit.TableName()).Where("model = ?", item.Model).Count(&count)
		if count > 0 {
			continue
		}

		audit.Version = item.Version
		audit.IsAuditing = 0
		audit.Model = item.Model
		if err := f5.GetApp().GetOrmDb(constant.CONF_DB).Create(audit).Error; err != nil {
			f5.GetSysLog().Error("添加audit 失败:%s, %d", item.Version, item.Model)
		}
	}
}

func (this *AuditApi) AuditList(c *gin.Context) {
	var auditList []system.Audit
	err := f5.GetApp().GetOrmDb(constant.CONF_DB).Find(&auditList).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
		"data":    auditList,
	})
}

func (this *AuditApi) AddAudit(c *gin.Context) {
	audit := system.Audit{}
	if err := c.ShouldBindJSON(&audit); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	var count int64
	db := f5.GetApp().GetOrmDb(constant.CONF_DB)
	db.Table(audit.TableName()).Where("model = ?", audit.Model).Count(&count)
	if count > 0 {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "无法再插入",
		})
		return
	}

	err := db.Create(&audit).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
	})
}

func (this *AuditApi) UpdateAudit(c *gin.Context) {
	idx, _ := strconv.ParseUint(c.Param("idx"), 10, 64)
	audit := system.Audit{}
	if err := c.ShouldBindJSON(&audit); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	var count int64
	db := f5.GetApp().GetOrmDb(constant.CONF_DB)
	db.Table(audit.TableName()).Where("idx = ?", idx).Count(&count)
	if count < 1 {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "不存在的数据",
		})
		return
	}

	// db.Table(audit.TableName()).Where("idx != ?", idx).Where("model = ?", audit.Model).Count(&count)
	// if count > 0 {
	// 	c.JSON(http.StatusOK, gin.H{
	// 		"code":    1,
	// 		"message": "已有同类型审核",
	// 	})
	// 	return
	// }

	err := db.Select("*").Omit("idx", "model").Where("idx = ?", idx).Updates(&audit).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
	})
}
