package system

import (
	"main/constant"
	"main/model/system"
	"f5"
	"github.com/gin-gonic/gin"
	"net/http"
)

type BattleServerApi struct{}

func (s *BattleServerApi) Add(c *gin.Context) {
	server := system.BattleServer{}
	if err := c.ShouldBindJSON(&server); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	err := f5.GetApp().GetOrmDb(constant.ACCOUNT_DB).Create(&server).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	// response
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
		"data":    "",
	})
}

func (s *BattleServerApi) List(c *gin.Context) {
	var battleServers []system.BattleServer
	err := f5.GetApp().GetOrmDb(constant.ACCOUNT_DB).Find(&battleServers).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
		"data":    battleServers,
	})
}

func (s *BattleServerApi) Update(c *gin.Context) {
	entity := system.BattleServer{}
	if err := c.ShouldBindJSON(&entity); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	var count int64
	f5.GetApp().GetOrmDb(constant.ACCOUNT_DB).Table(entity.TableName()).Where("idx = ?", entity.Idx).Count(&count)
	if count < 1 {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "不存在的数据",
		})
		return
	}

	err := f5.GetApp().GetOrmDb(constant.ACCOUNT_DB).Select("*").Omit("idx").Where("idx = ?", entity.Idx).Updates(&entity).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
	})
}
