package system

import (
	"f5"
	"main/common"
	"net/http"
	"q5"

	"github.com/gin-gonic/gin"
)

func checkparam(obj any, c *gin.Context) bool {
	if err := c.ShouldBindJSON(obj); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return false
	}

	return true
}

func query(dbname string, cursor int64, sql string, c *gin.Context, loadcall func(ds *f5.DataSet) interface{}) {
	rspObj := struct {
		Code    int32                   `json:"code"`
		Message string                  `json:"message"`
		Page    common.StreamPagination `json:"page"`
		Data    []interface{}           `json:"data"`
	}{
		Code:    0,
		Message: "success",
		Data:    []interface{}{},
	}

	pageSize := q5.AdjustRangeValue(q5.ToInt32(c.DefaultQuery("page_size", "")), 1, 20)

	params := []string{}
	subFilters := []f5.DbQueryFilter{}

	orderBy := ""

	f5.GetGoStyleDb().StreamPageQuery(
		dbname,
		pageSize,
		cursor,
		sql,
		params,
		f5.GetDbFilter().Comp(subFilters...),
		orderBy,
		func(err error, pagination *f5.StreamPagination) {
			rspObj.Page.FillPage(pagination)
		},
		func(ds *f5.DataSet) {
			rspObj.Data = append(rspObj.Data, loadcall(ds))
		})

	c.JSON(200, rspObj)
}
