package system

import (
	"encoding/json"
	"f5"
	"fmt"
	"main/constant"
	"main/model/system"
	"main/mt"
	"net/http"
	"q5"
	"sort"

	"github.com/gin-gonic/gin"
)

type GameSwitchApi struct {
}

var namescrpt = map[string]string{}

func (gs *GameSwitchApi) Init() {
	switches := []struct {
		Name string `json:"switch_name"`
		Desc string `json:"switch_desc"`
	}{}

	if jsonStr, err := f5.ReadJsonFile("../config/game_switch.json"); err == nil {
		if err := q5.DecodeJson(jsonStr, &switches); err != nil {
			panic(fmt.Sprintf("parse metafile error %s %s", "game_switch.json", err))
		}
	} else {
		panic(fmt.Sprintf("load metafile error %s %s", "game_switch.json", err))
	}

	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	for _, item := range switches {
		namescrpt[item.Name] = item.Desc
		if err := f5.GetApp().GetOrmDb(constant.CONF_DB).Where("switch_name = ?", item.Name).Take(new(system.GameSwitch)).Error; err == nil {
			continue
		}

		gswitch := new(system.GameSwitch)
		gswitch.Name = item.Name
		gswitch.Opened = 0
		gswitch.CreateTime = nowDaySeconds
		gswitch.ModifyTime = nowDaySeconds
		if err := f5.GetApp().GetOrmDb(constant.CONF_DB).Create(gswitch).Error; err != nil {
			f5.GetSysLog().Error("添加game switch 失败:%s, %s", item.Name, item.Desc)
		}
	}
}

func (this *GameSwitchApi) List(c *gin.Context) {
	pagesize := c.DefaultQuery("pagesize", "10")
	page := c.DefaultQuery("page", "")
	result := []struct {
		system.GameSwitch
		Servers []int32 `json:"serverList"`
		Remark  string  `json:"remark"`
	}{}
	f5.GetGoStyleDb().PageQuery(
		constant.CONF_DB,
		q5.ToInt32(pagesize),
		q5.ToInt32(page),
		"SELECT * FROM t_game_switch WHERE 1=1",
		[]string{},
		f5.GetDbFilter().Comp([]f5.DbQueryFilter{}...),
		"",
		func(err error, pg *f5.Pagination) {
			if err != nil {
				c.JSON(http.StatusOK, gin.H{
					"code":    1,
					"message": err.Error(),
				})
				return
			}

			for pg.Rows.Next() {
				p := q5.NewSliceElement(&result)
				p.Idx = q5.SafeToInt64(pg.Rows.GetByName("idx"))
				p.Name = pg.Rows.GetByName("switch_name")
				p.Opened = q5.SafeToInt32(pg.Rows.GetByName("is_open"))
				desc, exist := namescrpt[p.Name]
				if exist {
					p.Remark = desc
				}
				p.Servers = make([]int32, 0)
				json.Unmarshal([]byte(pg.Rows.GetByName("server_list")), &p.Servers)
				p.AuditOpen = q5.SafeToInt32(pg.Rows.GetByName("audit_is_open"))
				p.CreateTime = q5.SafeToInt32(pg.Rows.GetByName("createtime"))
				p.ModifyTime = q5.SafeToInt32(pg.Rows.GetByName("modifytime"))
			}
			c.JSON(http.StatusOK, gin.H{
				"code":       0,
				"message":    "获取成功",
				"data":       result,
				"total":      pg.Total,
				"total_page": pg.TotalPages,
				"cur_page":   pg.CurrentPage,
			})
		})
}

func (this *GameSwitchApi) Add(c *gin.Context) {
	req := struct {
		Name string `binding:"required" json:"switch_name"`
		Open *int32 `binding:"required" json:"is_open"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	if err := f5.GetApp().GetOrmDb(constant.CONF_DB).Where("switch_name = ?", req.Name).Take(new(system.GameSwitch)).Error; err == nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "已存在",
		})
		return
	}
	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	gswitch := new(system.GameSwitch)
	gswitch.Name = req.Name
	gswitch.Opened = *req.Open
	gswitch.CreateTime = nowDaySeconds
	gswitch.ModifyTime = nowDaySeconds
	if err := f5.GetApp().GetOrmDb(constant.CONF_DB).Create(gswitch).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "添加成功",
	})
}

func (this *GameSwitchApi) Edit(c *gin.Context) {
	req := struct {
		Name      string `binding:"required" json:"switch_name"`
		Open      *int32 `binding:"required" json:"is_open"`
		Servers   []int  `json:"serverList"`
		AuditOpen *int32 `binding:"required" json:"audit_is_open"`
		Remark    string `json:"remark"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	gswitch := new(struct {
		system.GameSwitch
		Servers string `gorm:"column:server_list" json:"serverList"`
	})
	db := f5.GetApp().GetOrmDb(constant.CONF_DB)
	if err := db.Take(gswitch, "switch_name = ?", req.Name).Error; err != nil {
		if !f5.IsOrmErrRecordNotFound(err) {
			c.JSON(http.StatusOK, gin.H{
				"code":    500,
				"message": "sever internal error:" + err.Error(),
			})
			return
		} else {
			c.JSON(http.StatusOK, gin.H{
				"code":    2,
				"message": "无法查到记录",
			})
			return
		}
	}

	sort.Ints(req.Servers)
	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	gswitch.Opened = *req.Open
	serverbytes, _ := json.Marshal(req.Servers)
	gswitch.Servers = string(serverbytes)
	gswitch.AuditOpen = *req.AuditOpen
	gswitch.ModifyTime = nowDaySeconds
	if err := db.Where("switch_name = ?", req.Name).Save(gswitch).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    500,
			"message": "sever internal error:" + err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "",
	})

	this.refreshAct(func(f5.HttpCliResponse) {})
}

func (this *GameSwitchApi) Del(c *gin.Context) {
	req := struct {
		Name string `binding:"required" json:"switch_name"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	gswitch := new(system.GameSwitch)
	db := f5.GetApp().GetOrmDb(constant.CONF_DB)
	if err := db.Take(gswitch, "switch_name = ?", req.Name).Error; err != nil {
		if !f5.IsOrmErrRecordNotFound(err) {
			c.JSON(http.StatusOK, gin.H{
				"code":    500,
				"message": "sever internal error:" + err.Error(),
			})
			return
		} else {
			c.JSON(http.StatusOK, gin.H{
				"code":    2,
				"message": "无法查到记录",
			})
			return
		}
	}

	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	gswitch.ModifyTime = nowDaySeconds
	if err := db.Where("switch_name = ?", req.Name).Delete(gswitch).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    500,
			"message": "sever internal error:" + err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "",
	})
}

func (this *GameSwitchApi) Refresh(c *gin.Context) {
	this.refreshAct(func(rsp f5.HttpCliResponse) {
		if rsp.GetErr() != nil {
			f5.RspErr2(c, 500, rsp.GetErr().Error())
			return
		}
		rspJson := struct {
			ErrCode int32  `json:"errcode"`
			ErrMsg  string `json:"errmsg"`
			CP      int32  `json:"contributionPoint"`
		}{}
		if q5.DecodeJson(rsp.GetRawData(), &rspJson) != nil {
			f5.RspErr2(c, 500, "server internal error")
			return
		}
		f5.RspErr2(c, rspJson.ErrCode, rspJson.ErrMsg)
	})
}

func (this *GameSwitchApi) refreshAct(cb func(f5.HttpCliResponse)) {
	f5.GetHttpCliMgr().SendGoStyleRequest(
		mt.Table.Config.RandElement().GetGameapiUrl()+"/webapp/index.php",
		map[string]string{
			"c": "OutAppTools",
			"a": "refreshServerSwitch",
		},
		cb)
}
