package system

import (
	"f5"
	"github.com/gin-gonic/gin"
	"github.com/xuri/excelize/v2"
	"main/constant"
	"main/model/system"
	"net/http"
	"q5"
)

type GroupMemberApi struct {
}

func (this *GroupMemberApi) List(c *gin.Context) {
	req := struct {
		GroupId int64          `binding:"required" json:"group_id,string" form:"group_id"`
		PageDto system.PageDto `json:"page_dto"`
	}{}
	if err := c.ShouldBindQuery(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	result := []*system.Member{}
	f5.GetGoStyleDb().PageQuery(
		constant.MAIL_DB,
		q5.ToInt32(req.PageDto.PageSize),
		q5.ToInt32(req.PageDto.Page),
		"SELECT * FROM t_member WHERE 1=1",
		[]string{},
		f5.GetDbFilter().Comp(
			f5.GetDbFilter().EQ("group_id", q5.ToString(req.GroupId)).And(),
		),
		"",
		func(err error, pg *f5.Pagination) {
			if err != nil {
				c.JSON(http.StatusOK, gin.H{
					"code":    1,
					"message": err.Error(),
				})
				return
			}

			for pg.Rows.Next() {
				p := new(system.Member)
				p.GroupId = q5.ToInt64(pg.Rows.GetByName("group_id"))
				p.MemberId = pg.Rows.GetByName("member_id")
				q5.AppendSlice(&result, p)
			}
			c.JSON(http.StatusOK, gin.H{
				"code":    0,
				"message": "获取成功",
				"data":    result,
				"total":   pg.Total,
			})
		})
}

func (this *GroupMemberApi) Add(c *gin.Context) {
	req := struct {
		GroupId  int64  `binding:"required" json:"group_id,string"`
		MemberId string `binding:"required" json:"member_id"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	if err := f5.GetApp().GetOrmDb(constant.MAIL_DB).Take(new(system.UserGroup), req.GroupId).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "组不存在",
		})
		return
	}
	if err := f5.GetApp().GetOrmDb(constant.MAIL_DB).Where("group_id =? and member_id =?", req.GroupId, req.MemberId).Take(new(system.Member)).Error; err == nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "数据存在",
		})
		return
	}
	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	member := new(system.Member)
	member.GroupId = req.GroupId
	member.MemberId = req.MemberId
	member.CreateTime = nowDaySeconds
	member.ModifyTime = nowDaySeconds
	if err := f5.GetApp().GetOrmDb(constant.MAIL_DB).Create(member).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "添加成功",
	})
}

func (this *GroupMemberApi) Edit(c *gin.Context) {
	req := struct {
		GroupId     int64  `binding:"required" json:"group_id,string"`
		MemberId    string `binding:"required" json:"member_id"`
		NewMemberId string `binding:"required" json:"new_member_id"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	member := new(system.Member)
	db := f5.GetApp().GetOrmDb(constant.MAIL_DB)
	if err := db.Take(member, "group_id =? and member_id =?", req.GroupId, req.MemberId).Error; err != nil {
		if !f5.IsOrmErrRecordNotFound(err) {
			c.JSON(http.StatusOK, gin.H{
				"code":    500,
				"message": "sever internal error:" + err.Error(),
			})
			return
		} else {
			c.JSON(http.StatusOK, gin.H{
				"code":    2,
				"message": "无法查到记录",
			})
			return
		}
	}
	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	member.GroupId = req.GroupId
	member.MemberId = req.NewMemberId
	member.ModifyTime = nowDaySeconds
	if err := db.Where("group_id =? and member_id =?", req.GroupId, req.MemberId).Save(member).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    500,
			"message": "sever internal error:" + err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "",
	})
}

func (this *GroupMemberApi) Del(c *gin.Context) {
	req := struct {
		GroupId  int64  `binding:"required" json:"group_id,string"`
		MemberId string `binding:"required" json:"member_id"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	member := new(system.Member)
	db := f5.GetApp().GetOrmDb(constant.MAIL_DB)
	if err := db.Take(member, "group_id =? and member_id =?", req.GroupId, req.MemberId).Error; err != nil {
		if !f5.IsOrmErrRecordNotFound(err) {
			c.JSON(http.StatusOK, gin.H{
				"code":    500,
				"message": "sever internal error:" + err.Error(),
			})
			return
		} else {
			c.JSON(http.StatusOK, gin.H{
				"code":    2,
				"message": "无法查到记录",
			})
			return
		}
	} else {
	}
	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	member.ModifyTime = nowDaySeconds
	if err := db.Where("group_id =? and member_id =?", req.GroupId, req.MemberId).Delete(member).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    500,
			"message": "sever internal error:" + err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "",
	})
}

func (this *GroupMemberApi) UploadExcel(c *gin.Context) {
	group_id := c.Request.FormValue("group_id")
	if group_id == "" {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "缺少参数",
		})
		return
	}
	file, _, err := c.Request.FormFile("file")
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "上传文件失败",
		})
		return
	}
	xlsx, err := excelize.OpenReader(file)
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "解析文件失败",
		})
		return
	}
	rows, err := xlsx.GetRows(xlsx.GetSheetName(xlsx.GetActiveSheetIndex()))
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	members := []*system.Member{}
	for i, row := range rows {
		if i == 0 {
			continue
		}
		member := new(system.Member)
		//member.GroupId = q5.ToInt64(row[0])
		member.GroupId = q5.ToInt64(group_id)
		member.MemberId = row[0]
		member.CreateTime = nowDaySeconds
		member.ModifyTime = nowDaySeconds
		members = append(members, member)
	}
	if err := f5.GetApp().GetOrmDb(constant.MAIL_DB).Create(members).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "添加成功",
	})
}
