package system

import (
	"f5"
	"fmt"
	"jccommon"
	"main/constant"
	"main/model/system"
	"main/mt"
	"net/http"
	"q5"
	"strings"

	"github.com/gin-gonic/gin"
)

type PlayerApi struct {
}

func (pai *PlayerApi) Info(c *gin.Context) {
	type InfoForm struct {
		Playername   string `json:"playername"`
		AccurateName bool   `json:"accurate_name"`
		Account_id   string `json:"account_id"`
		Address      string `json:"address"`
		Email        string `json:"email"`
	}

	reqJson := InfoForm{}
	if !checkparam(&reqJson, c) {
		return
	}

	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))

	filterstr := ""
	if reqJson.Playername != "" {
		if reqJson.AccurateName {
			filterstr = " name = '" + reqJson.Playername + "' "
		} else {
			filterstr = " name like '%" + reqJson.Playername + "%'"
		}

	} else if reqJson.Account_id != "" {
		filterstr = " account_id = '" + reqJson.Account_id + "'"
	} else if reqJson.Address != "" {
		filterstr = " address = '" + reqJson.Address + "'"
	} else if reqJson.Email != "" {
		email := strings.ToLower(reqJson.Email)
		sql := fmt.Sprintf(`SELECT account_id FROM t_immutable_account WHERE idx > 1 AND lower_case_email = '%s'`, email)

		accountids := []string{}
		f5.GetGoStyleDb().RawQuery(
			constant.ACCOUNT_DB,
			sql,
			[]string{},
			func(err error, ds *f5.DataSet) {
				if err != nil {
					return
				}

				for ds.Next() {
					accountids = append(accountids, ds.GetByIndex(0))
				}
			})

		if len(accountids) == 0 {
			f5.RspErr2(c, 0, "")
			return
		}

		filterstr = " account_id IN ("
		for _, id := range accountids {
			filterstr += "'" + id + "',"
		}
		filterstr = filterstr[:len(filterstr)-1]
		filterstr += ")"

	} else {
		f5.RspErr2(c, 1, "input one of playername, account_id, address, email")
		return
	}
	sql := fmt.Sprintf(`SELECT * FROM t_user WHERE idx > %d AND %s `, cursor, filterstr)

	query(constant.GAME_DB, cursor, sql, c, func(ds *f5.DataSet) interface{} {
		p := new(system.Player)
		p.LoadFromDs(ds)

		accountid := ""
		if reqJson.Playername != "" {
			if reqJson.AccurateName {
				accountid = p.Account_id
			}
		} else if reqJson.Account_id != "" {
			accountid = reqJson.Account_id
		} else if reqJson.Address != "" {
			accountid = p.Account_id
		} else if reqJson.Email != "" {
			p.Email = strings.ToLower(reqJson.Email)
		}

		if accountid != "" {
			sql := fmt.Sprintf(`SELECT lower_case_email FROM t_immutable_account WHERE idx > 1 AND account_id = '%s'`, accountid)

			f5.GetGoStyleDb().RawQuery(
				constant.ACCOUNT_DB,
				sql,
				[]string{},
				func(err error, ds *f5.DataSet) {
					if err != nil {
						return
					}

					if ds.Next() {
						p.Email = ds.GetByIndex(0)
					}
				})
		}

		return p
	})
}

func (pai *PlayerApi) BagQuery(c *gin.Context) {
	type BagQueryForm struct {
		Account_id string `binding:"required" json:"account_id"`
	}

	reqJson := BagQueryForm{}
	if !checkparam(&reqJson, c) {
		return
	}

	if len(reqJson.Account_id) > 0xFF {
		f5.RspErr2(c, 1, "输入过长")
		return
	}

	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))
	filterstr := " account_id = '" + reqJson.Account_id + "' "
	sql := fmt.Sprintf(`SELECT * FROM t_bag WHERE idx > %d AND %s `, cursor, filterstr)

	query(constant.GAME_DB, cursor, sql, c, func(ds *f5.DataSet) interface{} {
		p := new(system.BagItem)
		p.LoadFromDs(ds)
		return p
	})
}

func (pai *PlayerApi) HeroesQuery(c *gin.Context) {
	type HeroesQueryForm struct {
		Account_id string `binding:"required" json:"account_id"`
	}

	reqJson := HeroesQueryForm{}
	if !checkparam(&reqJson, c) || reqJson.Account_id == "" {
		return
	}

	if len(reqJson.Account_id) > 0xFF {
		f5.RspErr2(c, 1, "输入过长")
		return
	}

	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))
	filterstr := "account_id = '" + reqJson.Account_id + "' "
	{
		sql := fmt.Sprintf(`SELECT address FROM t_immutable_account WHERE idx > 1 AND account_id = '%s'`, reqJson.Account_id)

		address := ""
		f5.GetGoStyleDb().RawQuery(
			constant.ACCOUNT_DB,
			sql,
			[]string{},
			func(err error, ds *f5.DataSet) {
				if err != nil {
					return
				}

				if ds.Next() {
					address = ds.GetByIndex(0)
				}
			})

		if address != "" {
			lockaddrlist := []string{}
			mt.Table.Contract.Traverse(func(mtc *mt.Contract) bool {
				if mtc.GetName() == jccommon.CONTRACT_NAME_NFTLock {
					lockaddrlist = append(lockaddrlist, strings.ToLower(mtc.GetAddress()))
				}
				return true
			})
			sql = fmt.Sprintf(`SELECT token_id FROM t_nft WHERE idx > 1 AND  token_type IN (1, 12) AND (owner_address = '%s'`, address)
			if len(lockaddrlist) > 0 {
				sql += fmt.Sprintf(" OR (last_lock_address = '%s' AND owner_address IN (", address)
				for _, addr := range lockaddrlist {
					sql += "'" + addr + "',"
				}
				sql = sql[:len(sql)-1]
				sql += "))"
			}
			sql += ")"
			f5.GetSysLog().Debug("tokenid sql:%s", sql)
			tokenidlist := []string{}
			f5.GetGoStyleDb().RawQuery(
				constant.BCNFT_DB,
				sql,
				[]string{},
				func(err error, ds *f5.DataSet) {
					if err != nil {
						return
					}

					for ds.Next() {
						tokenidlist = append(tokenidlist, ds.GetByIndex(0))
					}
				})
			if len(tokenidlist) > 0 {
				tokenfilterstr := "OR token_id IN ("
				for _, v := range tokenidlist {
					tokenfilterstr += v + ","
				}
				tokenfilterstr = tokenfilterstr[:len(tokenfilterstr)-1]
				tokenfilterstr += ")"

				filterstr += tokenfilterstr
			}
		}
	}
	sql := fmt.Sprintf(`SELECT * FROM t_hero WHERE idx > %d AND (%s)`, cursor, filterstr)
	f5.GetSysLog().Debug("hero sql:%s", sql)

	query(constant.GAME_DB, cursor, sql, c, func(ds *f5.DataSet) interface{} {
		p := new(system.Hero)
		p.LoadFromDs(ds)
		return p
	})
}

func (pai *PlayerApi) GoldBullionQuery(c *gin.Context) {
	type GoldBullionQueryForm struct {
		Token_id     string `json:"token_id"`
		Open_address string `json:"open_address"`
	}

	reqJson := GoldBullionQueryForm{}
	if !checkparam(&reqJson, c) {
		return
	}

	filterstr := ""
	if reqJson.Token_id != "" {
		filterstr = " token_id = '" + reqJson.Token_id + "'"
	} else if reqJson.Open_address != "" {
		filterstr = " open_address = '" + reqJson.Open_address + "'"
	} else {
		f5.RspErr2(c, 1, "input one of token_id, open_address")
		return
	}

	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))
	sql := fmt.Sprintf(`SELECT * FROM t_gold_bullion WHERE idx > %d AND %s `, cursor, filterstr)

	query(constant.GAME_DB, cursor, sql, c, func(ds *f5.DataSet) interface{} {
		p := new(system.GoldBullion)
		p.LoadFromDs(ds)
		return p
	})
}

func (pai *PlayerApi) TicketConsumeQuery(c *gin.Context) {
	type TicketConsumeQueryForm struct {
		Room_id    string `binding:"required" json:"room_id"`
		Account_id string `json:"account_id"`
	}

	reqJson := TicketConsumeQueryForm{}
	if !checkparam(&reqJson, c) {
		return
	}

	filterstr := " match_room_uuid = '" + reqJson.Room_id + "' "
	if reqJson.Account_id != "" {
		filterstr += " AND account_id = '" + reqJson.Account_id + "' "
	}

	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))
	sql := fmt.Sprintf(`SELECT * FROM t_ticket_consume_record WHERE idx > %d AND %s `, cursor, filterstr)

	query(constant.GAME_DB, cursor, sql, c, func(ds *f5.DataSet) interface{} {
		p := new(system.TicketConsume)
		p.LoadFromDs(ds)
		return p
	})
}

func (pai *PlayerApi) GameMallQuery(c *gin.Context) {
	type GameMallQueryForm struct {
		Order_id       string `json:"order_id"`
		Seller         string `json:"seller"`
		Seller_address string `json:"seller_address"`
	}

	reqJson := GameMallQueryForm{}
	if !checkparam(&reqJson, c) {
		return
	}

	filterstr := ""
	if reqJson.Order_id != "" {
		filterstr = " order_id = '" + reqJson.Order_id + "'"
	} else if reqJson.Seller != "" {
		filterstr = " seller = '" + reqJson.Seller + "'"
	} else if reqJson.Seller_address != "" {
		filterstr = " seller_address = '" + reqJson.Seller_address + "'"
	} else {
		f5.RspErr2(c, 1, "input one of order_id, seller, seller_address")
		return
	}

	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))
	sql := fmt.Sprintf(`SELECT * FROM t_ingame_mall WHERE idx > %d AND %s `, cursor, filterstr)

	query(constant.GAME_DB, cursor, sql, c, func(ds *f5.DataSet) interface{} {
		p := new(system.MallItem)
		p.LoadFromDs(ds)
		return p
	})
}

func (pai *PlayerApi) InappOrderQuery(c *gin.Context) {
	type InappOrderQueryForm struct {
		Order_id    string `json:"order_id"`
		SP_Order_id string `json:"sp_order_id"`
		Account_id  string `json:"account_id"`
	}

	reqJson := InappOrderQueryForm{}
	if !checkparam(&reqJson, c) {
		return
	}

	filterstr := ""
	if reqJson.Order_id != "" {
		filterstr = " order_id = '" + reqJson.Order_id + "'"
	} else if reqJson.SP_Order_id != "" {
		filterstr = " sp_order_id = '" + reqJson.SP_Order_id + "'"
	} else if reqJson.Account_id != "" {
		filterstr = " account_id = '" + reqJson.Account_id + "'"
	} else {
		f5.RspErr2(c, 1, "input one of order_id, app store order_id, account_id")
		return
	}

	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))
	sql := fmt.Sprintf(`SELECT * FROM t_inapp_order WHERE idx > %d AND %s `, cursor, filterstr)

	query(constant.GAME_DB, cursor, sql, c, func(ds *f5.DataSet) interface{} {
		p := new(struct {
			Idx         int64   `gorm:"column:idx" json:"idx"`
			Order_id    string  `gorm:"column:order_id" json:"order_id"`
			SP_Order_id string  `gorm:"column:sp_order_id" json:"sp_order_id"`
			Platform    int     `gorm:"column:platform" json:"platform"`
			Account_id  string  `gorm:"column:account_id" json:"account_id"`
			Goods_id    int     `gorm:"column:goods_id" json:"goods_id"`
			Price       float64 `gorm:"column:price" json:"price"`
			Status      int     `gorm:"column:status" json:"status"`
			Createtime  int     `gorm:"column:createtime" json:"createtime"`
			Modifytime  int     `gorm:"column:modifytime" json:"modifytime"`
		})
		f5.UnmarshalModel(ds, p)
		return p
	})
}

func (pai *PlayerApi) RechargeQuery(c *gin.Context) {
	type RechargeQueryForm struct {
		Identity string `binding:"required" json:"identity"`
	}

	reqJson := RechargeQueryForm{}
	if !checkparam(&reqJson, c) {
		return
	}

	if len(reqJson.Identity) > 0xFF {
		f5.RspErr2(c, 1, "输入过长")
		return
	}

	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))
	filterstr :=
		" (receiver_account_id = '" + reqJson.Identity +
			"' OR account_address = '" + reqJson.Identity +
			"' OR passport_address = '" + reqJson.Identity +
			"' OR lower_case_email = '" + reqJson.Identity + "')"
	sql := fmt.Sprintf(`SELECT * FROM t_recharge_order WHERE idx > %d AND %s `, cursor, filterstr)

	query(constant.BCNFT_DB, cursor, sql, c, func(ds *f5.DataSet) interface{} {
		p := new(system.RechargeOrder)
		f5.UnmarshalModel(ds, p)
		return p
	})
}

func (pai *PlayerApi) VipBindQuery(c *gin.Context) {
	type VipBindQueryForm struct {
		Identity string `binding:"required" json:"identity"`
	}

	reqJson := VipBindQueryForm{}
	if !checkparam(&reqJson, c) {
		return
	}

	if len(reqJson.Identity) > 0xFF {
		f5.RspErr2(c, 1, "输入过长")
		return
	}

	rsp := []struct {
		Account  string `gorm:"column:account_address" json:"account_address"`
		Passport string `gorm:"column:passport_address" json:"passport_address"`
		Email    string `gorm:"column:lower_case_email" json:"lower_case_email"`
	}{}

	sql := `SELECT * FROM t_vip_bind WHERE idx > 0 AND (account_address = ? OR passport_address = ?)`
	f5.GetGoStyleDb().RawQuery(
		constant.BCNFT_DB,
		sql,
		[]string{
			reqJson.Identity,
			reqJson.Identity,
		},
		func(err error, ds *f5.DataSet) {
			if err != nil {
				f5.RspErr2(c, 500, "server internal error")
				return
			}

			for ds.Next() {
				p := q5.NewSliceElement(&rsp)
				f5.UnmarshalModel(ds, p)
			}

			c.JSON(http.StatusOK, gin.H{
				"code":    0,
				"message": "",
				"data":    rsp,
			})
		})

}

func (pai *PlayerApi) VipUserQuery(c *gin.Context) {
	type VipUserQueryForm struct {
		Account string `binding:"required" json:"account_address"`
	}

	reqJson := VipUserQueryForm{}
	if !checkparam(&reqJson, c) {
		return
	}

	if len(reqJson.Account) > 0xFF {
		f5.RspErr2(c, 1, "输入过长")
		return
	}

	rsp := []struct {
		Account      string `gorm:"column:account_address" json:"account_address"`
		Level        int64  `gorm:"column:vip_lv" json:"vip_lv"`
		Balance      string `gorm:"column:escec_balance" json:"escec_balance"`
		Stacking     string `gorm:"column:escec_stacking" json:"escec_stacking"`
		Convert      string `gorm:"column:escec_convert" json:"escec_convert"`
		Total        string `gorm:"column:escec_total" json:"escec_total"`
		StackingLast int64  `gorm:"column:stacking_last_src_idx" json:"stacking_last_src_idx"`
		TransferLast int64  `gorm:"column:escec_transfer_last_src_idx" json:"escec_transfer_last_src_idx"`
		DepositLast  int64  `gorm:"column:vester_deposit_last_src_idx" json:"vester_deposit_last_src_idx"`
		WithdrawLast int64  `gorm:"column:vester_withdraw_last_src_idx" json:"vester_withdraw_last_src_idx"`
	}{}

	sql := `SELECT * FROM t_vip_user WHERE idx > 0 AND account_address = ?`
	f5.GetGoStyleDb().RawQuery(
		constant.BCNFT_DB,
		sql,
		[]string{
			reqJson.Account,
		},
		func(err error, ds *f5.DataSet) {
			if err != nil {
				f5.RspErr2(c, 500, "server internal error")
				return
			}

			for ds.Next() {
				p := q5.NewSliceElement(&rsp)
				f5.UnmarshalModel(ds, p)
			}

			c.JSON(http.StatusOK, gin.H{
				"code":    0,
				"message": "",
				"data":    rsp,
			})
		})
}
