package system

import (
	"f5"
	. "main/global"
	"main/model/system"
	"main/mt"
	"net/http"
	"q5"
	"strings"

	"github.com/gin-gonic/gin"
)

type UserApi struct {
	MetaMaskApi
}

func (this *UserApi) Login(c *gin.Context) {
	if f5.IsTestEnv() {
		accountAddress := "0x0cf08c8661f87cbf823128d37ba75814cbcdf08b"
		s := GetSessionMgr().AddSession(accountAddress, accountAddress)
		c.JSON(http.StatusOK, gin.H{
			"code":    0,
			"message": "登录成功",
			//"data":    user,
			"token":   s.GetToken(),
		})
	}
}

func (this *UserApi) Info(c *gin.Context) {
	//token := c.Request.Header.Get("Authorization")
	//strArr := strings.Split(token, "|")
	//username := c.Query("username")
	//if username == "" {
	//	c.JSON(http.StatusOK, gin.H{
	//		"code":    1,
	//		"message": "请求参数为空",
	//	})
	//	return
	//}

	user := new(system.SysUser)
	user.Idx = 1000
	user.Username = ""
	user.Password = ""
	q5.NewSlice(&user.Roles, 0, 10)
	q5.AppendSlice(&user.Roles, "admin")
	s := GetSessionMgr().GetSessionByToken(c.Request.Header.Get("Authorization"))
	accountaddr := ""
	if s != nil {
		accountaddr = s.GetAccountAddress()
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
		"data":    user,
		"ui":      mt.Table.Permission.GetUIPermission(accountaddr),
	})

	/*
		user := system.SysUser{}
		err := f5.GetApp().GetOrmDb(constant.ADMIN_DB).Where("username = ?", strArr[0]).First(&user).Error
		if err != nil {
			c.JSON(http.StatusOK, gin.H{
				"code":    1,
				"message": "暂无此用户",
			})
			return
		}
		user.Roles = append(user.Roles, "admin")

		c.JSON(http.StatusOK, gin.H{
			"code":    0,
			"message": "success",
			"data":    user,
		})*/
}

func (this *UserApi) Logout(c *gin.Context) {
	//token := c.Request.Header.Get("Authorization")
	//strArr := strings.Split(token, "|")
	//GetApp().RemoveSession(strArr[0])
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "success",
	})
}

func (this *UserApi) MetaMaskLogin(c *gin.Context) {
	reqJson := struct {
		AccountAddress string `json:"account"`
		Nonce          string `json:"nonce"`
		Signature      string `json:"signature"`
		Tips           string `json:"tips"`
	}{}
	if err := c.ShouldBindJSON(&reqJson); err != nil {
		f5.RspErr2(c, 1, err.Error())
		return
	}
	if len(reqJson.Nonce) > 1024 || len(reqJson.Signature) > 1024 || len(reqJson.Tips) > 1024 {
		f5.RspErr2(c, 2, "param error")
		return
	}
	if !GetSessionMgr().IsValidNonce(reqJson.Nonce) {
		f5.RspErr2(c, 3, "param error")
		return
	}

	f5.GetHttpCliMgr().SendGoStyleRequest(
		mt.Table.Web3ServiceCluster.RandElement().GetUrl()+"/webapp/index.php",
		map[string]string{
			"c":         "BcService",
			"a":         "authVerifySignature",
			"tips":      reqJson.Tips,
			"nonce":     reqJson.Nonce,
			"signature": reqJson.Signature,
		},
		func(rsp f5.HttpCliResponse) {
			if rsp.GetErr() != nil {
				f5.RspErr2(c, 500, rsp.GetErr().Error())
				return
			}
			rspJson := struct {
				ErrCode   int32  `json:"errcode"`
				ErrMsg    string `json:"errmsg"`
				Recovered string `json:"recovered"`
			}{}
			if q5.DecodeJson(rsp.GetRawData(), &rspJson) != nil {
				f5.RspErr2(c, 500, "server internal error")
				return
			}
			if strings.ToLower(rspJson.Recovered) != strings.ToLower(reqJson.AccountAddress) {
				f5.RspErr2(c, 500, "server internal error")
				return
			}
			s := GetSessionMgr().AddSession(reqJson.AccountAddress, reqJson.Signature)
			accountaddr := s.GetAccountAddress()
			c.JSON(http.StatusOK, gin.H{
				"code":    0,
				"message": "success",
				"token":   s.GetToken(),
				"ui":      mt.Table.Permission.GetUIPermission(accountaddr),
			})
		})
}
