package system

import (
	"f5"
	"github.com/gin-gonic/gin"
	"main/constant"
	"main/model/system"
	"net/http"
)

type UserGroupApi struct {
}

func (this *UserGroupApi) List(c *gin.Context) {
	groups := []system.UserGroup{}
	f5.GetApp().GetOrmDb(constant.MAIL_DB).Find(&groups)
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "",
		"data":    groups,
	})
}

func (this *UserGroupApi) Add(c *gin.Context) {
	req := struct {
		GroupName string `binding:"required" json:"group_name"`
		GroupDesc string `json:"group_desc"`
		Enabled   int32  `json:"enabled"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	db := f5.GetApp().GetOrmDb(constant.MAIL_DB)
	if err := db.Where("group_name = ?", req.GroupName).Take(new(system.UserGroup)).Error; err == nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    2,
			"message": "组名已存在",
		})
		return
	} else {
		if !f5.IsOrmErrRecordNotFound(err) {
			c.JSON(http.StatusOK, gin.H{
				"code":    500,
				"message": "sever internal error:" + err.Error(),
			})
			return
		}
	}
	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	group := new(system.UserGroup)
	group.GroupId = f5.GetApp().NewLockNodeUuid()
	group.GroupName = req.GroupName
	group.GroupDesc = req.GroupDesc
	group.Enabled = req.Enabled
	group.CreateTime = nowDaySeconds
	group.ModifyTime = nowDaySeconds
	if err := db.Create(group).Error; err != nil {
		if !f5.IsOrmErrRecordNotFound(err) {
			c.JSON(http.StatusOK, gin.H{
				"code":    500,
				"message": "sever internal error:" + err.Error(),
			})
			return
		}
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "",
	})
}

func (this *UserGroupApi) Edit(c *gin.Context) {
	req := struct {
		GroupId   int64  `binding:"required" json:"group_id,string"`
		GroupName string `binding:"required" json:"group_name"`
		GroupDesc string `json:"group_desc"`
		Enabled   int32  `json:"enabled"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	group := new(system.UserGroup)
	db := f5.GetApp().GetOrmDb(constant.MAIL_DB)
	if err := db.Take(group, req.GroupId).Error; err != nil {
		if !f5.IsOrmErrRecordNotFound(err) {
			c.JSON(http.StatusOK, gin.H{
				"code":    500,
				"message": "sever internal error:" + err.Error(),
			})
			return
		} else {
			c.JSON(http.StatusOK, gin.H{
				"code":    2,
				"message": "无法查到记录",
			})
			return
		}
	} else {
	}
	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	group.GroupName = req.GroupName
	group.GroupDesc = req.GroupDesc
	group.Enabled = req.Enabled
	group.ModifyTime = nowDaySeconds
	if err := db.Save(group).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    500,
			"message": "sever internal error:" + err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "",
	})
}

func (this *UserGroupApi) Del(c *gin.Context) {
	req := struct {
		GroupId int64 `binding:"required" json:"group_id,string"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	group := new(system.UserGroup)
	db := f5.GetApp().GetOrmDb(constant.MAIL_DB)
	if err := db.Take(group, req.GroupId).Error; err != nil {
		if !f5.IsOrmErrRecordNotFound(err) {
			c.JSON(http.StatusOK, gin.H{
				"code":    500,
				"message": "sever internal error:" + err.Error(),
			})
			return
		} else {
			c.JSON(http.StatusOK, gin.H{
				"code":    2,
				"message": "无法查到记录",
			})
			return
		}
	} else {
	}
	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	group.ModifyTime = nowDaySeconds
	if err := db.Delete(group).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    500,
			"message": "sever internal error:" + err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "",
	})
}
