package system

import (
	"f5"
	"main/constant"
	"main/model/system"
	"net/http"
	"q5"

	"github.com/gin-gonic/gin"
	"github.com/xuri/excelize/v2"
)

type WhiteListApi struct {
}

func (bpa *WhiteListApi) List(c *gin.Context) {
	req := struct {
		PageDto system.PageDto `json:"page_dto"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	sql := "SELECT * FROM t_whitelist WHERE 1=1"
	params := []string{}	
	result := []*system.WhiteListItem{}
	f5.GetGoStyleDb().PageQuery(
		constant.CONF_DB,
		q5.ToInt32(req.PageDto.PageSize),
		q5.ToInt32(req.PageDto.Page),
		sql,
		params,
		f5.GetDbFilter().Comp([]f5.DbQueryFilter{}...),
		" ORDER BY user_identity ",
		func(err error, pg *f5.Pagination) {
			if err != nil {
				c.JSON(http.StatusOK, gin.H{
					"code":    1,
					"message": err.Error(),
				})
				return
			}

			for pg.Rows.Next() {
				p := new(system.WhiteListItem)
				p.Enable = q5.SafeToInt32(pg.Rows.GetByName("enable"))
				p.Identity = pg.Rows.GetByName("user_identity")
				p.CreateTime = q5.SafeToInt32(pg.Rows.GetByName("createtime"))
				p.ModifyTime = q5.SafeToInt32(pg.Rows.GetByName("modifytime"))
				q5.AppendSlice(&result, p)
			}
			c.JSON(http.StatusOK, gin.H{
				"code":       0,
				"message":    "获取成功",
				"data":       result,
				"total":      pg.Total,
				"total_page": pg.TotalPages,
				"cur_page":   pg.CurrentPage,
			})
		})
}

func (bpa *WhiteListApi) Add(c *gin.Context) {
	req := struct {
		Identity string `binding:"required" json:"user_identity"`
		Enable   int32  `json:"enable"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	info := new(system.WhiteListItem)
	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())

	var count int64 = 0
	db := f5.GetApp().GetOrmDb(constant.CONF_DB).Table("t_whitelist").Where("user_identity = ?", req.Identity)
	if err := db.Count(&count).Error; err == nil && count > 0 {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "数据存在",
		})
		return
	}

	info.Identity = req.Identity
	info.Enable = req.Enable
	info.CreateTime = nowDaySeconds
	info.ModifyTime = nowDaySeconds
	if err := f5.GetApp().GetOrmDb(constant.CONF_DB).Create(info).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "添加成功",
	})
}

func (bpa *WhiteListApi) Edit(c *gin.Context) {
	req := struct {
		Identity string `binding:"required" json:"user_identity"`
		Enable   int32  `json:"enable"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	whiteListItem := new(system.WhiteListItem)
	db := f5.GetApp().GetOrmDb(constant.CONF_DB)
	if err := db.Take(whiteListItem, "user_identity =?", req.Identity).Error; err != nil {
		if !f5.IsOrmErrRecordNotFound(err) {
			c.JSON(http.StatusOK, gin.H{
				"code":    500,
				"message": "sever internal error:" + err.Error(),
			})
			return
		} else {
			c.JSON(http.StatusOK, gin.H{
				"code":    2,
				"message": "无法查到记录",
			})
			return
		}
	}

	if whiteListItem.Enable != req.Enable {
		nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
		whiteListItem.Enable = req.Enable
		whiteListItem.ModifyTime = nowDaySeconds
		if err := db.Where("user_identity = ?", req.Identity).Save(whiteListItem).Error; err != nil {
			c.JSON(http.StatusOK, gin.H{
				"code":    500,
				"message": "sever internal error:" + err.Error(),
			})
			return
		}
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "",
	})
}

func (bpa *WhiteListApi) Del(c *gin.Context) {
	// req := struct {
	// 	Account string `binding:"required" json:"user_identity"`
	// 	Type    string `json:"type"`
	// }{}
	// if err := c.ShouldBindJSON(&req); err != nil {
	// 	c.JSON(http.StatusOK, gin.H{
	// 		"code":    1,
	// 		"message": err.Error(),
	// 	})
	// 	return
	// }

	// whiteListItem := new(system.WhiteListItem)
	// db := f5.GetApp().GetOrmDb(constant.CONF_DB).Where("user_identity = ?", req.Account)
	// if req.Type != "" {
	// 	db = db.Where("type = ?", req.Type)
	// }

	// if err := db.Take(whiteListItem).Error; err != nil {
	// 	if !f5.IsOrmErrRecordNotFound(err) {
	// 		c.JSON(http.StatusOK, gin.H{
	// 			"code":    500,
	// 			"message": "sever internal error:" + err.Error(),
	// 		})
	// 		return
	// 	} else {
	// 		c.JSON(http.StatusOK, gin.H{
	// 			"code":    2,
	// 			"message": "无法查到记录",
	// 		})
	// 		return
	// 	}
	// } else {
	// 	if whiteListItem.Deleted == 1 {
	// 		c.JSON(http.StatusOK, gin.H{
	// 			"code":    2,
	// 			"message": "无法查到记录",
	// 		})

	// 		return
	// 	}
	// }
	// nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	// whiteListItem.Deleted = 1
	// whiteListItem.ModifyTime = nowDaySeconds

	// if err := db.Save(whiteListItem).Error; err != nil {
	// 	c.JSON(http.StatusOK, gin.H{
	// 		"code":    500,
	// 		"message": "sever internal error:" + err.Error(),
	// 	})
	// 	return
	// }
	// c.JSON(http.StatusOK, gin.H{
	// 	"code":    0,
	// 	"message": "",
	// })
}

func (bpa *WhiteListApi) UploadExcel(c *gin.Context) {
	file, _, err := c.Request.FormFile("file")
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "上传文件失败",
		})
		return
	}
	xlsx, err := excelize.OpenReader(file)
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "解析文件失败",
		})
		return
	}
	rows, err := xlsx.GetRows(xlsx.GetSheetName(xlsx.GetActiveSheetIndex()))
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	whitelist := []*system.WhiteListItem{}
	for i, row := range rows {
		if i == 0 {
			continue
		}
		member := new(system.WhiteListItem)
		member.Identity = row[0]
		member.Enable = 1
		member.CreateTime = nowDaySeconds
		member.ModifyTime = nowDaySeconds
		whitelist = append(whitelist, member)
	}

	if err := f5.GetApp().GetOrmDb(constant.CONF_DB).Create(whitelist).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "",
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "添加成功",
	})
}

func (bpa *WhiteListApi) ListSuper(c *gin.Context) {
	req := struct {
		PageDto system.PageDto `json:"page_dto"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	sql := "SELECT * FROM t_super_whitelist WHERE 1=1"
	params := []string{}
	result := []system.SuperWhiteListItem{}
	f5.GetGoStyleDb().PageQuery(
		constant.CONF_DB,
		q5.ToInt32(req.PageDto.PageSize),
		q5.ToInt32(req.PageDto.Page),
		sql,
		params,
		f5.GetDbFilter().Comp([]f5.DbQueryFilter{}...),
		"",
		func(err error, pg *f5.Pagination) {
			if err != nil {
				c.JSON(http.StatusOK, gin.H{
					"code":    1,
					"message": err.Error(),
				})
				return
			}

			for pg.Rows.Next() {
				p := q5.NewSliceElement(&result)
				(*p).Identity = pg.Rows.GetByName("user_identity")
				(*p).Enable = q5.SafeToInt32(pg.Rows.GetByName("enable"))
				(*p).CreateTime = q5.SafeToInt32(pg.Rows.GetByName("createtime"))
				(*p).ModifyTime = q5.SafeToInt32(pg.Rows.GetByName("modifytime"))
			}
			c.JSON(http.StatusOK, gin.H{
				"code":       0,
				"message":    "获取成功",
				"data":       result,
				"total":      pg.Total,
				"total_page": pg.TotalPages,
				"cur_page":   pg.CurrentPage,
			})
		})
}

func (bpa *WhiteListApi) AddSuper(c *gin.Context) {
	req := struct {
		Identity string `binding:"required" json:"user_identity"`
		Enable   int32  `json:"enable"`
	}{}

	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	if req.Identity == "" || req.Enable < 0 || req.Enable > 1 {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "",
		})
		return
	}

	info := new(system.SuperWhiteListItem)

	var count int64 = 0
	db := f5.GetApp().GetOrmDb(constant.CONF_DB).Table(info.TableName()).Where("user_identity = ?", req.Identity)
	err := db.Count(&count).Error
	if err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "系统繁忙",
		})
		return
	} else if count > 0 {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "数据存在",
		})
		return
	}

	info.Identity = req.Identity
	info.Enable = req.Enable
	nowDaySeconds := int32(f5.GetApp().GetRealSeconds())
	info.CreateTime = nowDaySeconds
	info.ModifyTime = nowDaySeconds
	if err := f5.GetApp().GetOrmDb(constant.CONF_DB).Create(info).Error; err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "添加成功",
	})
}

func (bpa *WhiteListApi) EditSuper(c *gin.Context) {
	req := struct {
		Identity string `binding:"required" json:"user_identity"`
		Enable   int32  `json:"enable"`
	}{}
	if err := c.ShouldBindJSON(&req); err != nil {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": err.Error(),
		})
		return
	}

	if req.Identity == "" || req.Enable < 0 || req.Enable > 1 {
		c.JSON(http.StatusOK, gin.H{
			"code":    1,
			"message": "",
		})
		return
	}

	item := new(system.SuperWhiteListItem)
	db := f5.GetApp().GetOrmDb(constant.CONF_DB)
	if err := db.Take(item, "user_identity = ?", req.Identity).Error; err != nil {
		if !f5.IsOrmErrRecordNotFound(err) {
			c.JSON(http.StatusOK, gin.H{
				"code":    500,
				"message": "sever internal error:" + err.Error(),
			})
			return
		} else {
			c.JSON(http.StatusOK, gin.H{
				"code":    2,
				"message": "无法查到记录",
			})
			return
		}
	}

	if item.Enable != req.Enable {
		item.Enable = req.Enable
		item.ModifyTime = int32(f5.GetApp().GetRealSeconds())
		if err := db.Where("user_identity = ?", req.Identity).Save(item).Error; err != nil {
			c.JSON(http.StatusOK, gin.H{
				"code":    500,
				"message": "sever internal error:" + err.Error(),
			})
			return
		}
	}
	c.JSON(http.StatusOK, gin.H{
		"code":    0,
		"message": "",
	})
}
