package common

import (
	"f5"
	"q5"
)

type Attachment struct {
	ItemId  int32 `json:"itemid"`
	ItemNum int32 `json:"itemnum"`
}

type StreamPagination struct {
	NextCursor     string `json:"next_cursor"`
	PreviousCursor string `json:"previous_cursor"`
	Remaining      int32  `json:"remaining"`
}

type Session interface {
	GetAccountAddress() string
	GetToken() string
	GetLoginTime() int32
}

type SessionMgr interface {
	GenNonce() string
	IsValidNonce(string) bool
	AddSession(string, string) Session
	RemoveSssionWithToken(string)
	RemoveSssionWithAddress(string)
	GetSessionByToken(string) Session
	GetSessionByAddress(string) Session
}

type App interface {
	Run(func(), func())
}

func (this *StreamPagination) FillPage(page *f5.StreamPagination) {
	if page.NextCursor != 0 {
		this.NextCursor = q5.ToString(page.NextCursor)
	}
	if page.PreviousCursor != 0 {
		this.PreviousCursor = q5.ToString(page.PreviousCursor)
	}
	this.Remaining = page.Remaining
}
