package service

import (
	"f5"
	"fmt"
	"main/constant"
	"main/mt"
	"q5"
	"strings"
	"sync"
	"time"
)

type baseinfo struct {
	recharge int64
	balance  string
}

type addressBalance struct {
	gcTime   int64
	infoList q5.ConcurrentMap[string, *baseinfo] // <netid$toke_addr, baseinfo>
}

type chainbalance struct {
	gcTime int64

	infoList           q5.ConcurrentMap[string, *baseinfo]       // <netid$toke_addr, baseinfo>
	addressBalanceList q5.ConcurrentMap[string, *addressBalance] // <account, addressBalance>
}

func (this *chainbalance) init() {
	this.infoList = q5.ConcurrentMap[string, *baseinfo]{}
	this.addressBalanceList = q5.ConcurrentMap[string, *addressBalance]{}
	this.GetGlobalBalance("")
	go this.checkBalanceList()
}

func (this *chainbalance) unInit() {

}

func (this *chainbalance) GetAccoutBalance(netidtoken string, acc string) (string, int64) {
	info, exist := this.addressBalanceList.Load(acc)
	if !exist {
		tmpinfo := new(addressBalance)
		this.addressBalanceList.Store(acc, tmpinfo)
		info = &tmpinfo
	}

	if nowseconds := f5.GetApp().GetRealSeconds(); (*info).gcTime+60 < nowseconds {
		(*info).gcTime = nowseconds
		this.getChainBalance(acc)
	}

	v, exist := (*info).infoList.Load(netidtoken)
	if exist {
		return (*v).balance, (*v).recharge
	}

	return "", 0
}

func (this *chainbalance) GetGlobalBalance(netidtoken string) (string, int64) {
	if nowseconds := f5.GetApp().GetRealSeconds(); this.gcTime+60 < nowseconds {
		this.gcTime = nowseconds
		go this.getChainBalance("")
	}

	v, exist := this.infoList.Load(netidtoken)
	if exist {
		return (*v).balance, (*v).recharge
	}

	return "", 0
}

func (this *chainbalance) getChainBalance(acc string) {
	url := mt.Table.Web3ServiceCluster.RandElement().GetUrl() + "/webapp/index.php"
	var wg sync.WaitGroup
	mt.Table.Chain.Traverse(func(item *mt.Chain) bool {
		var info *baseinfo
		key := fmt.Sprintf("%d$%s", item.GetNetId(), item.GetTokenAddress())
		if acc == "" {
			tmp, exist := this.infoList.Load(key)
			if !exist {
				info = new(baseinfo)
				this.infoList.Store(key, info)
				tmp = &info
			} else {
				info = *tmp
			}
		} else {
			accinfo, exist := this.addressBalanceList.Load(acc)
			if !exist {
				tmp := new(addressBalance)
				this.addressBalanceList.Store(acc, tmp)
				accinfo = &tmp
				info = new(baseinfo)
				tmp.infoList.Store(key, info)
			} else {
				keyinfo, exist := (*accinfo).infoList.Load(key)
				if !exist {
					info = new(baseinfo)
					(*accinfo).infoList.Store(key, info)
				} else {
					info = *keyinfo
				}
			}
		}
		{
			sql := fmt.Sprintf(`SELECT sum(diamond) FROM t_recharge_order WHERE idx > 0 AND pay_status = 1 AND net_id = %d AND currency_name = '%s'`, item.GetNetId(), strings.ToLower(item.GetTokenAddress()))
			if acc != "" {
				sql = fmt.Sprintf(`%s AND account_address = '%s'`, sql, strings.ToLower(acc))
			}

			f5.GetGoStyleDb().RawQuery(
				constant.BCNFT_DB,
				sql,
				[]string{},
				func(err error, ds *f5.DataSet) {
					if err != nil {
						return
					}

					if ds.Next() {
						info.recharge = q5.SafeToInt64(ds.GetByIndex(0)) / 100
					}
				})

		}
		rpcurl := item.GetRpcUrl()

		doquery := func(account string, tokeaddr string, balance *string) {
			f5.GetHttpCliMgr().SendGoStyleRequest(
				url,
				map[string]string{
					"c":               "Erc20",
					"a":               "getBalance",
					"rpc_url":         rpcurl,
					"account_address": account,
					"token_address":   tokeaddr,
				},
				func(rsp f5.HttpCliResponse) {
					if rsp.GetErr() != nil {
						return
					}
					rspJson := struct {
						ErrCode int32  `json:"errcode"`
						ErrMsg  string `json:"errmsg"`
						Balance string `json:"balance"`
					}{}
					if q5.DecodeJson(rsp.GetRawData(), &rspJson) != nil {
						return
					}

					*balance = rspJson.Balance
				})
			wg.Done()
		}

		queryAcc := item.GetAccountAddress()
		if acc != "" {
			queryAcc = acc
		}
		wg.Add(1)
		go doquery(queryAcc, item.GetTokenAddress(), &info.balance)
		return true
	})

	wg.Wait()
}

func (this *chainbalance) checkBalanceList() {
	fmt.Println("checkBalanceList start")
	for {
		if time.Now().UTC().Hour() == 0 && time.Now().UTC().Minute() == 0 {
			nowseconds := f5.GetApp().GetRealSeconds()
			deletelist := []string{}
			this.addressBalanceList.Range(func(key string, value *addressBalance) bool {
				if value.gcTime+86400 < nowseconds {
					deletelist = append(deletelist, key)
				}
				return true
			})

			for _, account := range deletelist {
				this.addressBalanceList.Delete(account)
			}

			f5.GetSysLog().Info("delete chainbalance address cache count:%d", len(deletelist))
		}

		time.Sleep((time.Second * 60))
	}
}
