package middleware

import (
	"bytes"
	"errors"
	"f5"
	"io"
	"jccommon"
	"main/service"
	"net/http"
	net_url "net/url"
	"q5"
	"strings"

	"github.com/gin-gonic/gin"
	"github.com/google/uuid"
)

/*
转发规则
如果c是OutApp开头则不需要对account_id和session_id参数校验
其他的需要校验

OutApp不需要限制并发数!!!
Callback 也不需要
*/
func CaForward(c *gin.Context) {
	accountId := c.DefaultQuery("account_id", "")
	sessionId := c.DefaultQuery("session_id", "")

	cmd:=c.DefaultQuery("c", "")
	var needlimit bool = !strings.HasPrefix(cmd, "OutApp") && !strings.HasPrefix(cmd, "Callback")
	if needlimit && !jccommon.IsValidSessionId(accountId, sessionId) {
		f5.RspErr(c, 500, "invalid session_id")
		c.Abort()
		service.SApiForward.IncInvalidSessionTimes()
		return
	}

	if needlimit {
		cLock := service.SApiForward.AcquireLock(accountId)
		if cLock == nil {
			f5.RspErr(c, 500, "system busy")
			c.Abort()
			return
		}
		defer service.SApiForward.ReleaseLock(cLock)
	}
	
	service.SApiForward.IncTotalTimes()
	beginTick := q5.GetTickCount()
	defer func() {
		costTime := q5.GetTickCount() - beginTick
		service.SApiForward.UpdateCostTime(costTime)
	}()
	downStreamUrl, downStreamHost := service.SApiForward.GetDownStreamHost()
	newUrl := downStreamUrl + c.Request.URL.Path[5:]
	if !q5.StrContains(newUrl, "?") {
		newUrl = newUrl + "?"
	}
	params := []*[]string{}
	nonce := uuid.New().String()
	nowTime := f5.GetApp().GetRealSeconds()
	u := net_url.Values{}
	{
		for k, v := range c.Request.URL.Query() {
			u.Set(k, v[0])
			q5.AppendSlice(&params, &[]string{k, v[0]})
		}
		u.Set("__nonce", nonce)
		u.Set("__timestamp", q5.ToString(nowTime))
	}

	var httpRequest *http.Request
	var createErr error
	switch strings.ToUpper(c.Request.Method) {
	case "GET":
		{
			service.SApiForward.IncGetTimes()
			newUrl += u.Encode()
			httpRequest, createErr = http.NewRequest("GET", newUrl, nil)
			if !f5.IsOnlineEnv() {
				f5.GetSysLog().Info("CaForward method:%s newUrl:%s ", c.Request.Method, newUrl)
			}
		}
	case "POST":
		{
			service.SApiForward.IncPostTimes()
			if postData, err := c.GetRawData(); err == nil {
				newUrl += u.Encode()
				httpRequest, createErr = http.NewRequest("POST", newUrl, bytes.NewBuffer(postData))
				contentType := c.GetHeader("Content-Type")
				if contentType != "" {
					httpRequest.Header.Set("Content-Type", contentType)
				}
				if !f5.IsOnlineEnv() {
					f5.GetSysLog().Info("CaForward method:%s newUrl:%s Content-Type:%s postData:%s",
						c.Request.Method,
						newUrl,
						contentType,
						postData)
				}
			} else {
				createErr = err
			}
		}
	default:
		{
			createErr = errors.New("method error")
		}
	}
	if createErr != nil {
		service.SApiForward.IncCreateErrTimes()
		f5.RspErr(c, 500, "create request error")
		c.Abort()
		f5.GetSysLog().Info("CaForward create request url:%s error:%s", newUrl, createErr)
		return
	}
	if downStreamHost != "" {
		httpRequest.Host = downStreamHost
	}
	client := &http.Client{}
	if resp, err := client.Do(httpRequest); err == nil {
		defer resp.Body.Close()
		if bytes, err := io.ReadAll(resp.Body); err == nil {
			service.SApiForward.IncOkTimes()
			c.String(200, string(bytes))
			c.Abort()
			return
		} else {
			service.SApiForward.IncReadRspErrTimes()
			f5.RspErr(c, 500, "read response error")
			c.Abort()
			f5.GetSysLog().Info("CaForward read response url:%s eror:%s", newUrl, err)
			return
		}
	} else {
		service.SApiForward.IncDoErrTimes()
		f5.RspErr(c, 500, "client.Do error")
		c.Abort()
		f5.GetSysLog().Info("CaForward client.Do url:%s error:%s", newUrl, err)
		return
	}

}
