package task

import (
	"q5"
	"f5"
	"main/mt"
	"jccommon"
	"fmt"
	"main/constant"
	"main/service"
)

type goldBullionReturn struct {

}

func (this* goldBullionReturn) init() {
	go this.loadExpiredGoldBullion()
}

func (this* goldBullionReturn) unInit() {
}

func (this* goldBullionReturn) loadExpiredGoldBullion() {
	f5.GetGoStyleDb().LoopLoad(
		constant.GAME_DB,
		"goldBullionReturn",
		"t_gold_bullion",
		func () int64 {
			return 60 * 5
		},
		func (lastIdx int64) string {
			nowTime := f5.GetApp().GetRealSeconds()
			expiredTime := nowTime - 3600 * 24 - 3600
			sql := fmt.Sprintf(`
SELECT * FROM t_gold_bullion
WHERE idx > %d AND activated = 0 AND returned = 0 AND status=%d AND createtime < %d LIMIT 100`,
				lastIdx,
				jccommon.GOLD_BULLION_NO_OPEN,
				expiredTime)
			return sql
		},
		[]string{
		},
		func () int64 {
			return 10
		},
		func () int64 {
			return 60 * 5
		},
		this.returnGold)
}

func (this* goldBullionReturn) returnGold(ds *f5.DataSet) bool {
	netId := q5.ToInt32(ds.GetByName("net_id"))
	contractMeta := mt.Table.Contract.GetByNetIdName(netId, jccommon.CONTRACT_NAME_GoldBrick)
	if contractMeta == nil {
		return true
	}
	accountId := ds.GetByName("src_account_id")
	if accountId == "" {
		return true
	}
	contractAddress := contractMeta.GetAddress()
	tokenId := ds.GetByName("token_id")
	f5.GetSysLog().Info("returnGold net:%d accountId:%s contract_address:%s tokenId: %s",
		netId, accountId, contractAddress, tokenId)
	{
		rspObj := struct {
			Result struct {
				TokenId string `json:"token_id"`
			} `json:"result"`
		}{}
		url := fmt.Sprintf("%s/v1/chains/%s/collections/%s/nfts/%s",
			mt.Table.Config.GetById(0).GetImmutableBaseUrl(),
			mt.Table.Config.GetById(0).GetChainName(),
			contractAddress,
			tokenId)
		rsp, data, err := q5.HttpGetEx2(url, map[string]string{})
		if err != nil {
			f5.GetSysLog().Info("returnGold immutable api:%s request error:%s",
				url,
				err)
			return true
		}
		f5.GetSysLog().Info("returnGold immutable api:%s statusCode:%d body:%s",
			url,
			rsp.StatusCode,
			data)
		if rsp.StatusCode != 404 {
			return true
		}
		if rsp.StatusCode == 200 {
			if err := q5.DecodeJson(data, &rspObj); err != nil {
				f5.GetSysLog().Info("returnGold immutable api:%s decodeJson error:%s",
					url,
					err)
				return true
			}
			if rspObj.Result.TokenId == tokenId {
				f5.GetSysLog().Info("returnGold tokenId:%s exists",
					tokenId)
				return true
			}
		}

	}
	service.ReturnGoldBullion(accountId, netId, tokenId)
	return true
}
