package task

import (
	"q5"
	"f5"
	"fmt"
	"strings"
	"main/mt"
	"jccommon"
	"main/constant"
	"main/service"
	"math/rand"
)

type lockReturnValues struct {
	Nft string `json:"nft"`
	Sender string `json:"sender"`
	To string `json:"to"`
	TokenIds []string `json:"tokenIds"`
}

type nftLock struct {
	mailCfgHash *q5.ConcurrentMap[string, *jccommon.MailConfig]
}

func (this* nftLock) init() {
	this.mailCfgHash = new(q5.ConcurrentMap[string, *jccommon.MailConfig])
	this.registerMailConfig(constant.MAIL_GOLD_BULLION_LOCK,
		jccommon.MAIL_TAG1_GOLD_BULLION,
		jccommon.MAIL_TAG2_GOLD_BULLION_LOCK)
	go this.process()
}

func (this* nftLock) unInit() {
}

func (this* nftLock) process() {
	f5.GetGoStyleDb().LoopLoad(
		constant.BCEVENT_DB,
		"nftLock",
		"t_nft_lock_event",
		func () int64 {
			return 3
		},
		func (lastIdx int64) string {
			sql := fmt.Sprintf(`
SELECT * FROM t_nft_lock_event WHERE idx > %d AND (nft_sync_status = 0 OR nft_use_status = 0) LIMIT 100`,
				lastIdx,
			)
			return sql
		},
		[]string{
		},
		func () int64 {
			return 3
		},
		func () int64 {
			return 60 + q5.ToInt64(rand.Intn(3))
		},
		this.repair)
}

func (this* nftLock) repair(ds *f5.DataSet) bool {
	nftSyncStatus := q5.ToInt32(ds.GetByName("nft_sync_status"))
	nftUseStatus := q5.ToInt32(ds.GetByName("nft_use_status"))
	netId := q5.ToInt32(ds.GetByName("net_id"))
	idx := q5.ToInt64(ds.GetByName("idx"))
	lockTo := ds.GetByName("lock_to")
	lockSender := ds.GetByName("lock_sender")
	createTime := q5.ToInt32(ds.GetByName("createtime"))
	if nftSyncStatus == 0 && lockTo != "" {
		p := new(lockReturnValues)
		if q5.DecodeJson(ds.GetByName("return_values"), p) == nil {
			for _, val := range p.TokenIds {
				contractAddress := strings.ToLower(p.Nft)
				tokenId := val
				if service.NftExists(netId, contractAddress, tokenId) {
					service.NftUpdateLock(netId, contractAddress, tokenId, idx, lockTo, lockSender, createTime)
				}
			}
		}
	}
	if nftUseStatus == 0 {
		p := new(lockReturnValues)
		if q5.DecodeJson(ds.GetByName("return_values"), p) == nil {
			this.useNft(idx, netId, lockTo, p)
		}
	}
	return true
}

func (this *nftLock) useNft(dbIdx int64, netId int32, lockTo string, p *lockReturnValues) {
	lockTo = strings.ToLower(lockTo)
	contractAddress := strings.ToLower(p.Nft)

	goldMeta := mt.Table.Contract.GetByNetIdName(netId, jccommon.CONTRACT_NAME_GoldBrick)
	if goldMeta == nil {
		return
	}
	if contractAddress != strings.ToLower(goldMeta.GetAddress()) {
		f5.GetGoStyleDb().Update(
			constant.BCEVENT_DB,
			"t_nft_lock_event",
			[][]string {
				{"idx", q5.ToString(dbIdx)},
			},
			[][]string {
				{"nft_use_status", q5.ToString(1)},
			},
			func (err error, lastInsertId int64, rowsAffected int64) {
			})
		return
	}
	accountId := service.GetAccountIdByAddress(lockTo)
	if accountId == "" {
		return
	}
	ok := true
	for _, tokenId := range p.TokenIds {
		var itemId int32
		if service.GetGoldBullionByNetIdTokenId(netId, tokenId, &itemId) {
			if service.OpenGoldBullion(accountId, lockTo, netId, tokenId) {
				if this.internalSendMail(dbIdx, lockTo, constant.MAIL_GOLD_BULLION_LOCK, tokenId) {

				} else {
					ok = false
					break
				}
			} else {
				ok = false
				break
			}
		} else {
			ok = false
			break
		}
	}
	if ok {
		f5.GetGoStyleDb().Update(
			constant.BCEVENT_DB,
			"t_nft_lock_event",
			[][]string {
				{"idx", q5.ToString(dbIdx)},
				},
			[][]string {
				{"nft_use_status", q5.ToString(1)},
				},
			func (err error, lastInsertId int64, rowsAffected int64) {
			})
	}
}

func (this* nftLock) internalSendMail(dbIdx int64, accountAddress string, mailName string, tokenId string) bool {
	var itemId int32
	if !service.GetGoldBullionItemIdByTokenId(tokenId, &itemId) {
		return true
	}
	goldNum := jccommon.GetGoldBullionGoldNum(itemId)
	goldNumStr := this.formatGoldNum(goldNum)
	itemMeta := mt.Table.Item.GetById(q5.ToInt64(itemId))
	if itemMeta == nil {
		return true
	}
	accountId := service.GetAccountIdByAddress(accountAddress)
	if accountId == "" {
		return true
	}
	mailMeta := mt.Table.Mail.GetByName(mailName)
	if mailMeta == nil {
		return true
	}
	mailCfg := this.getMailConfig(mailName)
	if mailCfg == nil {
		return true
	}
	nowTime := f5.GetApp().GetRealSeconds()
	subject := mailMeta.GetTitle()
	content := mailMeta.ReplaceContent(map[string]string{
		"${item.name}": itemMeta.GetRealName(),
		"${gold}": goldNumStr,
	})
	uniKey := fmt.Sprintf("%s_%s_%d", mailName, tokenId, dbIdx)
	sendOk := service.SendSysMail(
		uniKey,
		accountId,
		subject,
		content,
		q5.ToInt32(nowTime),
		q5.ToInt32(nowTime) + 3600 * 24 * 7,
		mailCfg.Tag1,
		mailCfg.Tag2)
	return sendOk
}

func (this *nftLock) formatGoldNum(goldNum int32) string {
	if goldNum == 1000 {
		return "1,000"
	} else if goldNum == 10000 {
		return "10,000"
	}
	return q5.ToString(goldNum)
}

func (this *nftLock) getMailConfig(mailName string) *jccommon.MailConfig {
	if v, ok := this.mailCfgHash.Load(mailName); ok {
		return *v
	} else {
		return nil
	}
}

func (this *nftLock) registerMailConfig(mailName string, tag1 int32, tag2 int32) {
	p := new(jccommon.MailConfig)
	p.MailName = mailName
	p.Tag1 = tag1
	p.Tag2 = tag2
	this.mailCfgHash.Store(mailName, p)
}
