package common

import (
	"net"

	"cs"
	"f5"
	"ss"

	proto "github.com/golang/protobuf/proto"
)

type Room interface {
	GetRoomState() int32
	OnPlayerOffline(Player)
	OnPlayerOnline(Player)
}

type RoomMgr interface {
	ProcessCMMsg(*cs.CsNetMsgHandler, *f5.MsgHdr)
	RemoveRoomMember(*f5.MsgHdr)
}

type Player interface {
	GetRoom() Room
	SetRoom(Room)
	GetAccountId() string
	GetSessionId() string
	GetName() string
	GetAvatarUrl() string
	GetHeroId() string
	GetHeadFrame() string
	GetPing() int32
	SendMsg(proto.Message)
	IsOnline() bool
}

type PlayerMgr interface {
	ProcessCMMsg(*cs.CsNetMsgHandler, *f5.MsgHdr)
	GetPlayerBySocket(f5.WspCliConn) Player
	GetPlayerByAccountId(string) Player
	OnSocketClose(f5.WspCliConn)
}

type WspListener interface {
	ProcessSSMMsg(*ss.SsNetMsgHandler, *f5.MsgHdr)
	SendProxyMsg(net.Conn, uint16, proto.Message)
}

type HttpListener interface {
}

type App interface {
	Run(func(), func())
	AddNetMsg(*f5.MsgHdr)
}
