package room

import (
	"cs"
	"github.com/golang/protobuf/proto"
	"q5"
	"main/constant"
)

// TeamInfo from client data
type TeamInfo struct {
	TeamUUID   string       `json:"team_uuid"`
	MemberList []memberInfo `json:"member_list"`
}

type memberInfo struct {
	AccountID   string `json:"account_id"`
	Name        string `json:"name"`
	Avatar      string `json:"head_id"`
	AvatarFrame string `json:"head_frame"`
	HeroId      string `json:"hero_id"`
	IsLeader    int32  `json:"is_leader"`
	State       int32  `json:"is_ready"`
}

type team struct {
	room          *room
	teamId        int32
	teamUuid      string
	members       q5.ListHead
	obSortIdx     int32
	sortIdx       int32
	tmpTeamLeader *memberInfo
}

func (this *team) init(room *room, teamId int32, teamUuid string) {
	this.room = room
	this.teamId = teamId
	this.members.Init(nil)
	this.teamUuid = teamUuid
}

func (this *team) unInit() {
	deleteMembers := make([]*member, 0, this.members.Size())
	this.members.ForEach(
		func(data interface{}) bool {
			if mPtr, ok := data.(*member); ok {
				deleteMembers = append(deleteMembers, mPtr)
			}
			return true
		})
	for _, m := range deleteMembers {
		m.unInit()
	}
	delete(this.room.teamUuidHash, this.teamUuid)
	delete(this.room.teamIdHash, this.teamId)
	delete(this.room.obTeamUuidHash, this.teamUuid)
	delete(this.room.obTeamIdHash, this.teamId)
	this.room = nil
}

func (this *team) fillMFTeam(pb *cs.MFTeam) {
	pb.TeamId = proto.Int32(this.teamId)
	pb.TeamUuid = proto.String(this.teamUuid)
	q5.NewSlice(&pb.Members, 0, 4)
	this.members.ForEach(
		func(data interface{}) bool {
			m := data.(*member)
			if this.tmpTeamLeader != nil && m.hum.GetAccountId() == this.tmpTeamLeader.AccountID {
				m.isLeader = this.tmpTeamLeader.IsLeader
				m.state = this.tmpTeamLeader.State
				this.tmpTeamLeader = nil
			}

			pbM := &cs.MFMember{}
			m.fillMFMember(pbM)
			q5.AppendSlice(&pb.Members, pbM)
			return true
		})

	if this.tmpTeamLeader != nil && this.getTeamLeader() == nil {
		pbM := &cs.MFMember{
			AccountId: proto.String(this.tmpTeamLeader.AccountID),
			Name:      proto.String(this.tmpTeamLeader.Name),
			AvatarUrl: proto.String(this.tmpTeamLeader.Avatar),
			HeroId:    proto.String(this.tmpTeamLeader.HeroId),
			HeadFrame: proto.String(this.tmpTeamLeader.AvatarFrame),
			State:     proto.Int32(this.tmpTeamLeader.State),
			IsLeader:  proto.Int32(this.tmpTeamLeader.IsLeader),
		}
		q5.AppendSlice(&pb.Members, pbM)
	}
}

func (this *team) getTeamLeader() *member {
	var leader *member = nil
	this.members.ForEach(
		func(data interface{}) bool {
			m := data.(*member)
			if m.isLeader == 1 {
				leader = m
				return true
			}
			return false
		})
	return leader
}

func (this *team) addMember(m *member) {
	m.team = this
	this.members.AddTail(&m.teamEntry)
}

func (this *team) getMemberNum() int32 {
	return this.members.Size()
}

func (this *team) hasAlreadMember() bool {
	ok := false
	this.members.ForEach(
		func(data interface{}) bool {
			m := data.(*member)
			if m.state == MEMBER_READY_STATE {
				ok = true
				return false
			}
			return true
		})
	return ok
}

func (this *team) getOwnerCandidate() *member {
	var ownerCandidate *member
	this.members.ForEach(
		func(data interface{}) bool {
			m := data.(*member)
			if m.hum.IsOnline() {
				ownerCandidate = m
				return false
			}
			return true
		})
	return ownerCandidate
}

func (this *team) SaveTeamLeader(teamInfo *TeamInfo) {
	if this.tmpTeamLeader != nil {
		return
	}
	for _, mInfo := range teamInfo.MemberList {
		if mInfo.IsLeader == 1 {
			this.tmpTeamLeader = &memberInfo{
				AccountID:   mInfo.AccountID,
				Name:        mInfo.Name,
				Avatar:      mInfo.Avatar,
				AvatarFrame: mInfo.AvatarFrame,
				HeroId:      mInfo.HeroId,
				IsLeader:    mInfo.IsLeader,
				State:       mInfo.State,
			}
		}
	}
}

func (this *team) isObserver() bool {
	return this.room.getObTeamById(this.teamId) != nil
}

func (this *team) isFull() bool {
	return this.getMemberNum() >= constant.ROOM_MAX_TEAM_MEMBER_NUM
}

func newTeam(room *room, teamId int32, teamUuid string, leader *member) *team {
	t := new(team)
	leader.isLeader = 1
	t.init(room, teamId, teamUuid)
	t.addMember(leader)
	return t
}
