package cache

import (
	"q5"
	"f5"
	"cs"
	"github.com/golang/protobuf/proto"
	. "main/global"
)

type userProfile struct {
	accountId string
	name string
	avatarUrl string
	head string
	star int32
	totalKills int32
	totalWinTimes int32
	rank int32
	lastLoginTime int32
	lastSyncTime int64
}

func (this *userProfile) fillMFUser(pbUser *cs.MFUser) {
	pbUser.AccountId = proto.String(this.accountId)
	pbUser.Username = proto.String(this.name)
	pbUser.Avatar = proto.Int32(q5.ToInt32(this.avatarUrl))
	pbUser.AvatarHead = proto.Int32(q5.ToInt32(this.head))
	{
		guild := GetGuildMgr().GetGuildByAccountId(this.accountId)
		if guild != nil {
			pbUser.GuildId = proto.Int64(q5.ToInt64(guild.GetGuildId()))
			pbUser.GuildName = proto.String(guild.GetGuildName())
		}
	}
	pbUser.Rank = proto.Int32(this.rank)
	{
		pbUser.Battling = proto.Int32(0)
		hum := GetPlayerMgr().GetPlayerByAccountId(this.accountId)
		if hum != nil {
			if hum.IsOnline() {
				pbUser.OnlineStatus = proto.Int32(1)
			}
			if hum.IsBattling() {
				pbUser.Battling = proto.Int32(1)
			}
		} else {
			pbUser.OnlineStatus = proto.Int32(0)
		}
	}
	pbUser.LastLoginTime = proto.Int32(this.lastLoginTime)
}

func (this *userProfile) fillMFGuildMember(pbMember *cs.MFGuildMember) {
	pbMember.AccountId = proto.String(this.accountId)
	pbMember.Username = proto.String(this.name)
	pbMember.Avatar = proto.Int32(q5.ToInt32(this.avatarUrl))
	pbMember.AvatarHead = proto.Int32(q5.ToInt32(this.head))
	pbMember.Rank = proto.Int32(this.rank)
	{
		pbMember.Battling = proto.Int32(0)
		hum := GetPlayerMgr().GetPlayerByAccountId(this.accountId)
		if hum != nil {
			if hum.IsOnline() {
				pbMember.OnlineStatus = proto.Int32(1)
			}
			if hum.IsBattling() {
				pbMember.Battling = proto.Int32(1)
			}
		} else {
			pbMember.OnlineStatus = proto.Int32(0)
		}
	}
	pbMember.LastLoginTime = proto.Int32(this.lastLoginTime)
	pbMember.Level = proto.Int32(GetGuildMgr().GetGuildJob(this.accountId))
	pbMember.Star = proto.Int32(this.star)
}

func (this *userProfile) loadFromDb(ds *f5.DataSet) {
	this.accountId = ds.GetByName("account_id")
	this.name = ds.GetByName("name")
	this.avatarUrl = ds.GetByName("head_id")
	this.head = ds.GetByName("head")
	this.lastLoginTime = q5.ToInt32(ds.GetByName("last_login_time"))
	this.lastSyncTime = f5.GetApp().GetNowMillis()
}

func newUserProfile() *userProfile {
	p := new(userProfile)
	return p
}
