package common

import (
	"net"
	"ss"
	"cs"
	"f5"

	proto "github.com/golang/protobuf/proto"
)

type HistorySeasons struct {
	TotalKills int32 `json:"total_kills"`
	WinTimes   int32 `json:"win_times"`
}

type HeadRsp struct {
	Errcode int32    `json:"errcode"`
	Errmsg  string `json:"errmsg"`
}

type LoginRsp struct {
	Errcode int32    `json:"errcode"`
	Errmsg  string `json:"errmsg"`
	Info    struct {
		Activated      string           `json:"activated"`
		AccountID      string           `json:"account_id"`
		Name           string           `json:"name"`
		Avatar         string           `json:"head_id"`
		AvatarHead     string           `json:"head_frame"`
		Star           string           `json:"current_star_num"`
		Rank           string           `json:"current_rank"`
		LastLoginTime  string           `json:"last_login_time"`
		HistorySeasons []HistorySeasons `json:"history_seasons"`
	} `json:"info"`
}

type UserQueryStatusRsp struct {
	Errcode int    `json:"errcode"`
	Errmsg  string `json:"errmsg"`
	Users []*UserStatus
}

type UserStatus struct {
	AccountId string `json:"account_id"`
	Battling int32 `json:"battling"`
	Online int32 `json:"online"`
}

type WspListener interface {
	ProcessSSMMsg(*ss.SsNetMsgHandler, *f5.MsgHdr)
	SendProxyMsg(net.Conn, uint16, proto.Message)
}

type App interface {
	Run(func(), func())
	AddNetMsg(*f5.MsgHdr)
}

type Player interface {
	GetAccountId() string
	GetSessionId() string
	SendMsg(proto.Message)
	IsOnline() bool
	IsBattling() bool
}

type PlayerMgr interface {
	ProcessCMMsg(*cs.CsNetMsgHandler, *f5.MsgHdr)
	GetPlayerBySocket(f5.WspCliConn) Player
	GetPlayerByAccountId(string) Player
	OnSocketClose(f5.WspCliConn)
	GetRecommendPlayers(map[string]int32, map[string]int32, int32)
}

type FriendMgr interface {
	IsFriend(string, string) bool
	GetFriendList(string) []string
	GetBlackList(string) []string
	AsyncGetApplyList(int64, string, func(int32, string, int64, []string))
	AsyncApplyFriend(string, string, func(int32, string))
	AsyncAcceptApply(string, string, func(int32, string))
	AsyncRejectApply(string, string, func(int32, string))
	AsyncDeleteFriend(string, string, func(int32, string))
	AsyncAddBlack(string, string, func(int32, string))
	AsyncRemoveBlack(string, string, func(int32, string))
	AsyncHasApply(string, func(int32, string, int32))
}

type Guild interface {
	GetGuildId() string
	GetGuildName() string
	AsyncFillMFGuild(*cs.MFGuild, func(int32, string))
}

type GuildMgr interface {
	GetGuildByGuildId(string) Guild
	GetGuildByAccountId(string) Guild
	GetGuildByGuildName(string) Guild
	GetGuildJob(string) int32

	AsyncCreateGuild(string, string, int32, string, func(int32, string, string))
	AsyncGetApplyList(int64, string, func(int32, string, int64, []string))
	AsyncApplyJoin(string, string, func(int32, string, Guild))
	AsyncAcceptApply(string, string, func(int32, string, string, string))
	AsyncRejectApply(string, string, func(int32, string))
	AsyncGetGuildRank(int32, func(int32, string, []string))
	AsyncGetRecommendGuild(int32, func(int32, string, []string))

	AsyncLeave(string, func(int32, string, string, string, []string))
	AsyncKickout(string, string, func(int32, string, string, string))
	AsyncDisband(string, func(int32, string, []string))
	AsyncSetGuildJob(string, string, int32, func(int32, string, string, string))
	AsyncSearch(int64, string, func(int32, string, int64, []string))
	AsyncGetGuildLogs(string, string, func(int32, string))
	AsyncUpdateGuild(string, string, map[int32]string, func(int32, string))
	AsyncHasApply(string, func(int32, string, int32))

	NotifyGuildMsg(string, proto.Message)
}

type CacheMgr interface {
	PreLoadUsers([]string)
	AsyncGetUsersAndFillMFUser([]string, *[]*cs.MFUser, func(int32, string))
	AsyncGetUsersAndFillMFGuildMember([]string, *[]*cs.MFGuildMember, func(int32, string))
	AsyncSearch(int64, string, *[]*cs.MFUser, func(int32, string, int64))
	AsyncRecommendList(int32, string, *[]*cs.MFUser, func(int32, string))
}

type DbLogMgr interface {
	FriendApply(string, string)
	FriendAdd(string, string)
	FriendDel(string, string)

	GuildApply(string, string)
	GuildClearEmptyStart(string, string)
	GuildClearEmptyEnd(string, string)
	GuildCreateConsumeBegin(string, string, string)
	GuildCreateConsumeEnd(string, string, string, int32)
	GuildAccpetApply(string, string, string)
	GuildLeave(string, string)
	GuildKickout(string, string, string)
	GuildSetJob(string, string, string, int32)
	GuildDisband(string, string, int32)
	GuildRenameConsumeBegin(string, string, string)
	GuildRenameConsumeEnd(string, string, string, int32)
}
