package dblog

import (
	"q5"
	"f5"
	"main/constant"
)

type dbLogMgr struct {

}

const (
	LOG_TYPE_FRIEND_APPLY = "friend.apply"
	LOG_TYPE_FRIEND_ADD = "friend.add"
	LOG_TYPE_FRIEND_DEL = "friend.del"

	LOG_TYPE_GUILD_APPLY = "guild.apply"

	LOG_TYPE_GUILD_CLEAR_EMPTY_START = "guild.clear_empty.start"
	LOG_TYPE_GUILD_CLEAR_EMPTY_END = "guild.clear_empty.end"

	LOG_TYPE_GUILD_CREATE_CONSUME_BEGIN = "guild.create_consume.begin"
	LOG_TYPE_GUILD_CREATE_CONSUME_END = "guild.create_consume.end"

	LOG_TYPE_GUILD_ACCEPT_APPLY = "guild.accept"

	LOG_TYPE_GUILD_LEAVE = "guild.leave"
	LOG_TYPE_GUILD_KICKOUT = "guild.kickout"
	LOG_TYPE_GUILD_SETJOB = "guild.setjob"
	LOG_TYPE_GUILD_DISBAND = "guild.disband"

	LOG_TYPE_GUILD_RENAME_CONSUME_BEGIN = "guild.rename_consume.begin"
	LOG_TYPE_GUILD_RENAME_CONSUME_END = "guild.rename_consume.end"

)

func (this* dbLogMgr) Init() {

}

func (this* dbLogMgr) UnInit() {

}

func (this *dbLogMgr) addLog(fields [][]string) {
	nowTime := f5.GetApp().GetNowSeconds()
	q5.AppendSlice(&fields, []string{"createtime", q5.ToString(nowTime)})
	q5.AppendSlice(&fields, []string{"modifytime", q5.ToString(nowTime)})
	f5.GetJsStyleDb().Insert(
		constant.FRIEND_DB,
		"t_log",
		fields,
		func (error, int64, int64) {

		})
}

func (this *dbLogMgr) FriendApply(senderId string, targetId string) {
	this.addLog([][]string{
		{"log_type", LOG_TYPE_FRIEND_APPLY},
		{"param1", senderId},
		{"param2", targetId},
	})
}

func (this *dbLogMgr) FriendAdd(senderId string, targetId string) {
	this.addLog([][]string{
		{"log_type", LOG_TYPE_FRIEND_ADD},
		{"param1", senderId},
		{"param2", targetId},
	})
}

func (this *dbLogMgr) FriendDel(senderId string, targetId string) {
	this.addLog([][]string{
		{"log_type", LOG_TYPE_FRIEND_DEL},
		{"param1", senderId},
		{"param2", targetId},
	})
}

func (this *dbLogMgr) GuildApply(senderId string, guildId string) {
	this.addLog([][]string{
		{"log_type", LOG_TYPE_GUILD_APPLY},
		{"param1", senderId},
		{"param2", guildId},
	})
}

func (this *dbLogMgr) GuildClearEmptyStart(guildId string, guildName string) {
	this.addLog([][]string{
		{"log_type", LOG_TYPE_GUILD_CLEAR_EMPTY_START},
		{"param1", guildId},
		{"param2", guildName},
	})
}

func (this *dbLogMgr) GuildClearEmptyEnd(guildId string, guildName string) {
	this.addLog([][]string{
		{"log_type", LOG_TYPE_GUILD_CLEAR_EMPTY_END},
		{"param1", guildId},
		{"param2", guildName},
	})
}

func (this *dbLogMgr) GuildCreateConsumeBegin(accountId string, guildId string, guildName string) {
	this.addLog([][]string{
		{"log_type", LOG_TYPE_GUILD_CREATE_CONSUME_BEGIN},
		{"param1", accountId},
		{"param2", guildId},
		{"param3", guildName},
	})
}

func (this *dbLogMgr) GuildCreateConsumeEnd(accountId string, guildId string, guildName string, errCode int32) {
	this.addLog([][]string{
		{"log_type", LOG_TYPE_GUILD_CREATE_CONSUME_END},
		{"param1", accountId},
		{"param2", guildId},
		{"param3", guildName},
		{"paramr", q5.ToString(errCode)},
	})
}

func (this *dbLogMgr) GuildAccpetApply(guildId string, accountId string, targetId string) {
	this.addLog([][]string{
		{"log_type", LOG_TYPE_GUILD_ACCEPT_APPLY},
		{"param1", guildId},
		{"param2", accountId},
		{"param3", targetId},
	})
}

func (this *dbLogMgr) GuildLeave(guildId string, memberId string) {
	this.addLog([][]string{
		{"log_type", LOG_TYPE_GUILD_LEAVE},
		{"param1", guildId},
		{"param2", memberId},
	})
}

func (this *dbLogMgr) GuildKickout(guildId string, ownerId string, memberId string) {
	this.addLog([][]string{
		{"log_type", LOG_TYPE_GUILD_KICKOUT},
		{"param1", guildId},
		{"param2", ownerId},
		{"param3", memberId},
	})
}

func (this *dbLogMgr) GuildSetJob(guildId string, ownerId string, memberId string, guildJob int32) {
	this.addLog([][]string{
		{"log_type", LOG_TYPE_GUILD_SETJOB},
		{"param1", guildId},
		{"param2", ownerId},
		{"param3", memberId},
		{"param4", q5.ToString(guildJob)},
	})
}

func (this *dbLogMgr) GuildDisband(guildId string, sender string, reason int32) {
	this.addLog([][]string{
		{"log_type", LOG_TYPE_GUILD_DISBAND},
		{"param1", guildId},
		{"param2", sender},
		{"param3", q5.ToString(reason)},
	})
}

func (this *dbLogMgr) GuildRenameConsumeBegin(accountId string, guildId string, newGuildName string) {
	this.addLog([][]string{
		{"log_type", LOG_TYPE_GUILD_RENAME_CONSUME_BEGIN},
		{"param1", accountId},
		{"param2", guildId},
		{"param3", newGuildName},
	})
}


func (this *dbLogMgr) GuildRenameConsumeEnd(accountId string, guildId string, newGuildName string, errCode int32) {
	this.addLog([][]string{
		{"log_type", LOG_TYPE_GUILD_RENAME_CONSUME_END},
		{"param1", accountId},
		{"param2", guildId},
		{"param3", newGuildName},
		{"param4", q5.ToString(errCode)},
	})
}
