package task

import (
	"q5"
	"f5"
	"main/mt"
	"jccommon"
	"main/constant"
	"main/service"
	"fmt"
	"strings"
)

type chainActivity struct {

}

type lockReturnValues struct {
	Nft string `json:"nft"`
	Sender string `json:"sender"`
	To string `json:"to"`
	TokenIds []string `json:"tokenIds"`
}

type unLockReturnValues struct {
	Nft string `json:"nft"`
	User string `json:"user"`
	NftList [][]interface{} `json:"nftList"`
}

func (this* chainActivity) init() {
	mt.Table.Contract.Traverse(func (ele *mt.Contract) bool {
		if ele.GetName() == jccommon.CONTRACT_NAME_NFTLock {
			go this.processLockAndUnLockEvent(ele.GetNetId(), ele.GetAddress())
		}
		return true
	})
}

func (this* chainActivity) unInit() {
}

func (this* chainActivity) processLockAndUnLockEvent(netId int32, lockContractAddress string) {
	key := fmt.Sprintf("chainActivity.lastIdx.%d.%s", netId, lockContractAddress)
	if dbLastIdxStr, err := jccommon.GetDbParam(constant.BCEVENT_DB, key); err == nil {
		f5.GetGoStyleDb().IncrementLoad(
			constant.BCEVENT_DB,
			"chainActivity." + q5.ToString(netId) + "." + lockContractAddress,
			"t_blockchain_event",
			q5.ToInt64(dbLastIdxStr),
			func (lastIdx int64, maxIdx int64) (string, []string) {
				sql := fmt.Sprintf(`
SELECT * FROM t_blockchain_event
WHERE idx > %d AND idx <= %d AND net_id = %d AND contract_address = ? AND event_name IN (?, ?)
ORDER BY idx LIMIT 1000
`,
					lastIdx,
					maxIdx,
					netId)
				params := []string{
					lockContractAddress,
					"Lock",
					"UnLock",
				}
				return sql, params
			},
			func (newLastIdx int64) {
				jccommon.SaveDbParam(constant.BCEVENT_DB, key, q5.ToString(newLastIdx))
			},
			this.saveToDb)
	} else {
		panic(fmt.Sprintf("chainActivity.processLockAndUnLockEvent getDBParam error %s", err))
	}
}

func (this* chainActivity) saveToDb(ds *f5.DataSet) bool {
	f5.GetSysLog().Info("load lock/unlock idx:%s contract_name:%s event_name:%s",
		ds.GetByName("idx"),
		ds.GetByName("contract_name"),
		ds.GetByName("event_name"))
	idx := ds.GetByName("idx")
	txhash := ds.GetByName("txhash")
	logIndex := ds.GetByName("log_index")
	netId := ds.GetByName("net_id")
	eventName := ds.GetByName("event_name")
	contractAddress := ds.GetByName("contract_address")
	returnValues := ds.GetByName("return_values")
	createTime := ds.GetByName("createtime")
	modifyTime := ds.GetByName("modifytime")

	genWhereKv := func (tokenId string) [][]string {
		return [][]string{
			{"txhash", txhash},
			{"log_index", logIndex},
			{"net_id", netId},
			{"event_name", eventName},
			{"contract_address", contractAddress},
			{"token_id", tokenId},
		}
	}
	genInsertKv := func (nftAddress, tokenId, sender, to string, isMint int32) [][]string {
		var itemId int32
		var quality int32
		service.GetNftItemIdQuality(q5.ToInt32(netId), contractAddress, tokenId, &itemId, &quality)
		return [][]string{
			{"txhash", txhash},
			{"log_index", logIndex},
			{"net_id", netId},
			{"event_name", eventName},
			{"contract_address", strings.ToLower(contractAddress)},
			{"nft_contract_address", strings.ToLower(nftAddress)},
			{"token_id", tokenId},
			{"src_idx", idx},
			{"sender_address", strings.ToLower(sender)},
			{"to_address", strings.ToLower(to)},
			{"item_id", q5.ToString(itemId)},
			{"quality", q5.ToString(quality)},
			{"is_mint", q5.ToString(isMint)},
			{"createtime", createTime},
			{"modifytime", modifyTime},
		}
	}
	decodeJsonOk := false
	if eventName == "Lock" {
		p := new(lockReturnValues)
		if q5.DecodeJson(returnValues, p) == nil {
			decodeJsonOk = true
			for _, tokenId := range p.TokenIds {
				var dbErr error
				f5.GetGoStyleDb().Upsert(
					constant.BCEVENT_DB,
					"t_chain_activity",
					genWhereKv(tokenId),
					[][]string{},
					genInsertKv(p.Nft, tokenId, p.Sender, p.To, 0),
					func (err error, lastInsertId int64, rowsAffected int64) {
						dbErr = err
					})
				if dbErr != nil {
					return false
				}
			}
		}
	} else if eventName == "UnLock" {
		p := new(unLockReturnValues)
		if q5.DecodeJson(returnValues, p) == nil {
			sender := q5.SafeToString(p.User)
			decodeJsonOk = true
			for _, nft := range p.NftList {
				if len(nft) != 3 {
					decodeJsonOk = false
					break
				}
				tokenId := q5.SafeToString(nft[0])
				to := q5.SafeToString(nft[1])
				var isMint int32
				if value, ok := nft[2].(bool); ok {
					if value {
						isMint = 1
					}
				} else {
					decodeJsonOk = false
					break
				}
				var dbErr error
				f5.GetGoStyleDb().Upsert(
					constant.BCEVENT_DB,
					"t_chain_activity",
					genWhereKv(tokenId),
					[][]string{},
					genInsertKv(p.Nft, tokenId, sender, to, isMint),
					func (err error, lastInsertId int64, rowsAffected int64) {
						dbErr = err
					})
				if dbErr != nil {
					return false
				}
			}
		}
	} else {
		panic(fmt.Sprintf("chainActivity unknow event_name %s", eventName))
	}
	if !decodeJsonOk {
		jccommon.AddDbLog(constant.BCEVENT_DB, "chainActivity", "decodeJsonError",
			map[string]string{
				"param1": txhash,
				"param2": netId,
				"param3": eventName,
				"param4": idx,
				"param5": contractAddress,
			})
	}
	return true
}
