package task

import (
	"q5"
	"f5"
	"jccommon"
	"main/mt"
	"main/constant"
	"main/service"
	"strings"
	"fmt"
)

type esCecTransfer struct {

}

func (this *esCecTransfer) init() {
	mt.Table.Contract.Traverse(func (ele *mt.Contract) bool {
		if ele.GetName() == jccommon.CONTRACT_NAME_EsCec {
			go this.process(ele.GetNetId(), ele.GetAddress())
		}
		return true
	})
}

func (this *esCecTransfer) unInit() {
}

func (this *esCecTransfer) process(netId int32, contractAddress string) {
	key := fmt.Sprintf("esCecTransfer.lastIdx.%d.%s", netId, contractAddress)
	if dbLastIdxStr, err := jccommon.GetDbParam(constant.BCEVENT_DB, key); err == nil {
		f5.GetGoStyleDb().IncrementLoad(
			constant.BCEVENT_DB,
			"esCecTransfer." + q5.ToString(netId) + "." + contractAddress,
			"t_blockchain_event",
			q5.ToInt64(dbLastIdxStr),
			func (lastIdx int64, maxIdx int64) (string, []string) {
				sql := fmt.Sprintf(`
SELECT * FROM t_blockchain_event
WHERE idx > %d AND idx <= %d AND net_id = %d AND contract_address = ? AND event_name IN (?)
ORDER BY idx LIMIT 1000
`,
					lastIdx,
					maxIdx,
					netId)
				params := []string{
					contractAddress,
					"Transfer",
				}
				return sql, params
			},
			func (newLastIdx int64) {
				jccommon.SaveDbParam(constant.BCEVENT_DB, key, q5.ToString(newLastIdx))
			},
			this.saveToDb)
	} else {
		panic(fmt.Sprintf("esCecTransfer.Transfer getDBParam error %s", err))
	}
}

func (this* esCecTransfer) saveToDb(ds *f5.DataSet) bool {
	ro := jccommon.NewErc20TransferPo()
	bo := jccommon.NewBlockDataPo()
	if err := q5.DecodeJson(ds.GetByName("return_values"), ro); err != nil {
		f5.GetSysLog().Warning("esCecTransfer.parser.return_values error:%s", err)
		return false
	}
	if err := q5.DecodeJson(ds.GetByName("block_data"), bo); err != nil {
		f5.GetSysLog().Warning("esCecTransfer.parser.block_data error:%s", err)
		return false
	}
	idx := q5.ToInt64(ds.GetByName("idx"))
	if !q5.IsWeb3ZeroAddress(ro.To) {
		if !service.Vip.Add(ro.To, ro.Value, idx, "escec_balance", "escec_transfer_last_src_idx") {
			return false
		}
	}
	if !q5.IsWeb3ZeroAddress(ro.From) {
		if !service.Vip.Add(ro.From, "-" + ro.Value, idx, "escec_balance", "escec_transfer_last_src_idx") {
			return false
		}
	}
	txhash := ds.GetByName("txhash")
	logIndex := ds.GetByName("log_index")
	netId := ds.GetByName("net_id")
	eventName := ds.GetByName("event_name")
	contractAddress := ds.GetByName("contract_address")
	createTime := ds.GetByName("createtime")
	modifyTime := ds.GetByName("modifytime")

	if err, _, _ := f5.GetGoStyleDb().NewUpsert(
		constant.BCEVENT_DB,
		"t_escec_transfer",
		[][]string{
			{"txhash", txhash},
			{"log_index", logIndex},
			{"net_id", netId},
			{"contract_address", contractAddress},
		},
		[][]string{

		},
		[][]string{
			{"txhash", txhash},
			{"log_index", logIndex},
			{"net_id", netId},
			{"contract_address", strings.ToLower(contractAddress)},
			{"src_idx", q5.ToString(idx)},
			{"event_name", eventName},
			{"chain_timestamp", q5.ToString(bo.TimeStamp)},
			{"from_address", strings.ToLower(ro.From)},
			{"to_address", strings.ToLower(ro.To)},
			{"value", ro.Value},
			{"createtime", createTime},
			{"modifytime", modifyTime},
		}); err != nil {
		return false
	}
	return true
}
