package task

import (
	"q5"
	"f5"
	"main/mt"
	"main/constant"
	"main/service"
	"jccommon"
	"fmt"
	"strings"
)

type vester struct {

}

func (this *vester) init() {
	mt.Table.Contract.Traverse(func (ele *mt.Contract) bool {
		if ele.GetName() == jccommon.CONTRACT_NAME_Vester {
			go this.process(ele.GetNetId(), ele.GetAddress())
		}
		return true
	})
}

func (this *vester) unInit() {
}

func (this *vester) process(netId int32, contractAddress string) {
	key := fmt.Sprintf("vester.depositAndWithdraw.lastIdx.%d.%s", netId, contractAddress)
	if dbLastIdxStr, err := jccommon.GetDbParam(constant.BCEVENT_DB, key); err == nil {
		f5.GetGoStyleDb().IncrementLoad(
			constant.BCEVENT_DB,
			"vester.depositAndWithdraw." + q5.ToString(netId) + "." + contractAddress,
			"t_blockchain_event",
			q5.ToInt64(dbLastIdxStr),
			func (lastIdx int64, maxIdx int64) (string, []string) {
				sql := fmt.Sprintf(`
SELECT * FROM t_blockchain_event
WHERE idx > %d AND idx <= %d AND net_id = %d AND contract_address = ? AND event_name IN (?, ?)
ORDER BY idx LIMIT 1000
`,
					lastIdx,
					maxIdx,
					netId)
				params := []string{
					contractAddress,
					constant.VESTER_DEPOSIT_EVENT_NAME,
					constant.VESTER_WITHDRAW_EVENT_NAME,
				}
				return sql, params
			},
			func (newLastIdx int64) {
				jccommon.SaveDbParam(constant.BCEVENT_DB, key, q5.ToString(newLastIdx))
			},
			this.saveToDb)
	} else {
		panic(fmt.Sprintf("vester.depositAndWithdraw getDBParam error %s", err))
	}
}

func (this* vester) saveToDb(ds *f5.DataSet) bool {
	f5.GetSysLog().Info("load vester deposit/withdraw idx:%s contract_name:%s event_name:%s",
		ds.GetByName("idx"),
		ds.GetByName("contract_name"),
		ds.GetByName("event_name"))
	idx := ds.GetByName("idx")
	txhash := ds.GetByName("txhash")
	logIndex := ds.GetByName("log_index")
	netId := ds.GetByName("net_id")
	eventName := ds.GetByName("event_name")
	contractAddress := ds.GetByName("contract_address")
	blockData := ds.GetByName("block_data")
	returnValues := ds.GetByName("return_values")
	createTime := ds.GetByName("createtime")
	modifyTime := ds.GetByName("modifytime")

	blockObj := new(jccommon.BlockDataPo)
	if q5.DecodeJson(blockData, blockObj) != nil {
		fmt.Sprintf("vester error blockData %s\n", blockData)
		return false
	}
	whereKv := [][]string{
		{"txhash", txhash},
		{"log_index", logIndex},
		{"net_id", netId},
		{"event_name", eventName},
		{"contract_address", contractAddress},
	}
	genInsertKv := func (extKv [][]string) [][]string {
		insertKv := [][]string{
			{"txhash", txhash},
			{"log_index", logIndex},
			{"net_id", netId},
			{"event_name", eventName},
			{"contract_address", strings.ToLower(contractAddress)},
			{"src_idx", idx},
			{"chain_timestamp", q5.ToString(blockObj.TimeStamp)},
			{"createtime", createTime},
			{"modifytime", modifyTime},
		}
		insertKv = append(insertKv, extKv...)
		return insertKv
	}

	decodeJsonOk := false
	if eventName == constant.VESTER_DEPOSIT_EVENT_NAME {
		p := new(jccommon.VesterDepositPo)
		if q5.DecodeJson(returnValues, p) == nil {
			decodeJsonOk = true
			err, _, _ := f5.GetGoStyleDb().NewUpsert(
				constant.BCEVENT_DB,
				"t_vester_deposit_withdraw",
				whereKv,
				[][]string{},
				genInsertKv([][]string{
					{"account_address", strings.ToLower(p.Account)},
					{"deposit_amount", p.Amount},
				}))
			if err != nil {
				return false
			}
			return service.Vip.RecalcEsCecConvert(p.Account) == nil
		}
	} else if eventName == constant.VESTER_WITHDRAW_EVENT_NAME {
		p := new(jccommon.VesterWithdrawPo)
		if q5.DecodeJson(returnValues, p) == nil {
			decodeJsonOk = true
			err, _, _ := f5.GetGoStyleDb().NewUpsert(
				constant.BCEVENT_DB,
				"t_vester_deposit_withdraw",
				whereKv,
				[][]string{},
				genInsertKv([][]string{
					{"account_address", strings.ToLower(p.Account)},
					{"withdraw_clamied_amount", p.ClaimedAmount},
					{"withdraw_balance", p.Balance},
				}))
			if err != nil {
				return false
			}
			return service.Vip.RecalcEsCecConvert(p.Account) == nil
		}
	} else {
		panic(fmt.Sprintf("vester unknow event_name %s", eventName))
	}

	if !decodeJsonOk {
		jccommon.AddDbLog(constant.BCEVENT_DB, "vester.depositAndWithdraw", "decodeJsonError",
			map[string]string{
				"param1": txhash,
				"param2": netId,
				"param3": eventName,
				"param4": idx,
				"param5": contractAddress,
			})
	}

	return true
}
