package common

import (
	"github.com/gin-gonic/gin"
)

type AttachmentDto struct {
	ItemId int32 `json:"itemid"`
	ItemNum int32 `json:"itemnum"`
}

type MailDto struct {
	MailId string `json:"mailid"`
	From string `json:"from"`
	To string `json:"to"`
	Subject string `json:"subject"`
	Content string `json:"content"`
	Flags int32 `json:"flags"`
	SendTime int32 `json:"sendtime"`
	ExpireTime int32 `json:"expiretime"`
	MailType int32 `json:"mailtype"`
	MailSubType int32 `json:"mailsubtype"`
	Ext string `json:"ext"`
	Attachments []*AttachmentDto `json:"attachments"`
}

type App interface {
	Run(func(), func())
}

type Player interface {
	Lock() bool
	UnLock()
	GetAccountId() string
	GetRegisterTime() int32

	MarkMails([]Mail) error
	GetAttachment([]Mail, *gin.Context)
	DeleteMails([]Mail) error
	IsReadable(Mail) bool
	IsUnread(Mail) bool
}

type PlayerMgr interface {
	GetPlayerByAccountId(string) Player
	ForceCreatePlayer(string, string) Player
}

type Mail interface {
	GetMailId() int64
	IsValid(Player) bool
	HasAttachment() bool
	GetExpireTime() int32
	TraverseAttachment(cb func(int32, int32))
}

type MailMgr interface{
	CaGetMailList(c *gin.Context)
	CaMarkMail(c *gin.Context)
	CaGetUnreadMailCnt(c *gin.Context)
	CaGetAttachment(c *gin.Context)
	CaDeleteMails(c *gin.Context)
	HasTask() bool
}
