package activity

import (
	"f5"
	"fmt"
	"main/constant"
	"main/service"
	"q5"
	"strings"

	"github.com/gin-gonic/gin"
)

type ContriApi struct {
}

func (cta *ContriApi) HistoryQuery(c *gin.Context) {
	account := strings.ToLower(c.Param("account_address"))
	if account == "" {
		f5.RspErr(c, 2, "empty account")
		return
	}

	type info struct {
		Type int32  `json:"type"`
		Time int32  `json:"give_time"`
		CP   string `json:"contribution_point"`
	}

	rspObj := struct {
		Errcode int32   `json:"errcode"`
		Msg     int32   `json:"errmsg"`
		CP      string  `json:"contributionPoint"`
		Rows    []*info `json:"rows"`
	}{}

	mycp, _ := service.Contribution.GetAddressContribution(account, false)
	rspObj.CP = fmt.Sprintf("%.2f", mycp)
	q5.NewSlice(&rspObj.Rows, 0, 10)

	if mycp > 0.000001 {
		sql := `SELECT * FROM t_staking_daily_settlement WHERE idx > 0 and account_address = ?`
		f5.GetGoStyleDb().RawQuery(
			constant.BCNFT_DB,
			sql,
			[]string{account},
			func(err error, ds *f5.DataSet) {
				if err != nil {
					return
				}

				for ds.Next() {
					p := new(info)
					p.Type = 1
					p.Time = q5.SafeToInt32(ds.GetByName("settle_date"))
					p.CP = fmt.Sprintf("%.2f", q5.ToFloat64(ds.GetByName("contribution")))
					q5.AppendSlice(&rspObj.Rows, p)
				}
			})

		{
			loweremail, accountid := service.Contribution.GetEmailAccountId(account)
			if accountid != "" {
				sql = `SELECT * FROM t_contribution_history WHERE idx > 0 AND account_id = ?`
				f5.GetGoStyleDb().RawQuery(
					constant.GAME_DB,
					sql,
					[]string{accountid},
					func(err error, ds *f5.DataSet) {
						if err != nil {
							return
						}

						for ds.Next() {
							p := new(info)
							p.Type = 2
							p.Time = q5.SafeToInt32(ds.GetByName("createtime"))
							p.CP = fmt.Sprintf("%.2f", q5.ToFloat64(ds.GetByName("contribution")))
							q5.AppendSlice(&rspObj.Rows, p)
						}
					})
			}

			sql = `SELECT * FROM t_recharge_return_contribution WHERE idx > 0 AND user_identity IN (?`

			params := []string{account}
			if loweremail != "" {
				sql += ", ?"
				params = append(params, loweremail)
			}
			sql += ")"

			f5.GetGoStyleDb().RawQuery(
				constant.BCNFT_DB,
				sql,
				params,
				func(err error, ds *f5.DataSet) {
					if err != nil {
						return
					}

					for ds.Next() {
						p := new(info)
						p.Type = 3
						p.Time = q5.SafeToInt32(ds.GetByName("pay_time"))
						p.CP = fmt.Sprintf("%.2f", q5.ToFloat64(ds.GetByName("return_contribution")))
						q5.AppendSlice(&rspObj.Rows, p)
					}
				})
		}
	}

	q5.Sort(rspObj.Rows, func(a *info, b *info) bool {
		return a.Time > b.Time
	})

	c.JSON(200, rspObj)
}

func (cta *ContriApi) CECQuery(c *gin.Context) {
	account := strings.ToLower(c.Param("account_address"))
	if account == "" {
		f5.RspErr(c, 2, "empty account")
		return
	}

	rspObj := struct {
		Errcode int32  `json:"errcode"`
		Msg     int32  `json:"errmsg"`
		CP      string `json:"contributionPoint"`
		Info    struct {
			MyCP  string `json:"my_contribution"`
			GCP   string `json:"global_contribution"`
			GCEC  int64  `json:"total_cec_pool"`
			MyCEC int64  `json:"my_expected_cec"`
		} `json:"info"`
	}{}

	mycp, _ := service.Contribution.GetAddressContribution(account, false)
	totalgcp, _ := service.Contribution.GetGlobalContribution(false)
	rspObj.CP = fmt.Sprintf("%.2f", mycp)
	rspObj.Info.MyCP = rspObj.CP
	rspObj.Info.GCP = fmt.Sprintf("%.2f", totalgcp)
	rspObj.Info.GCEC = 500000
	rspObj.Info.MyCEC = q5.SafeToInt64(float64(rspObj.Info.GCEC) * (mycp) / totalgcp)

	c.JSON(200, rspObj)
}
