package asset

import (
	"q5"
	"f5"
	"main/constant"
	"main/common"
	"jccommon"
	"fmt"
	"main/mt"
	"strings"
	. "main/global"
	"github.com/gin-gonic/gin"
)

type AssetApi struct {

}

/*
   filterType: 0:全部 1:已上架 2:未上架 3:质押中
 */
func (this *AssetApi) AccountAsset(c *gin.Context) {
	pageSize := q5.AdjustRangeValue(q5.ToInt32(c.DefaultQuery("page_size", "")), 1, 20)
	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))
	netId := q5.ToInt64(c.Param("net_id"))
	accountAddress := strings.ToLower(c.Param("account_address"))
	filterType := q5.ToInt32(c.DefaultQuery("type", ""))
	searchName := c.DefaultQuery("search_name", "")
	sql := fmt.Sprintf(`
SELECT A.idx, A.net_id, A.token_type, A.token_id, A.contract_address, B.status FROM t_nft A LEFT JOIN t_order B
ON A.net_id = B.net_id AND A.contract_address = B.contract_address AND A.token_id = B.token_id
WHERE A.idx > %d AND A.net_id = %d AND A.owner_address=? `,
		cursor, netId)
	params := []string{accountAddress}
	if filterType == 1 {
		sql += " AND B.status = '" + jccommon.ORDER_STATUS_ACTIVE + "' "
	} else if filterType == 2 {
		sql += " AND (B.status IS NULL OR B.status <> '" + jccommon.ORDER_STATUS_ACTIVE + "') "
	} else if filterType == 3 {
		lockMeta := mt.Table.Contract.GetByNetIdName(int32(netId), jccommon.CONTRACT_NAME_NFTLock)
		if lockMeta != nil {
			sql += " AND A.last_lock_address = ? AND A.token_type <> ?"
			params = []string{
				strings.ToLower(lockMeta.GetAddress()),
				accountAddress,
				q5.ToString(jccommon.NFT_TYPE_GOLD_BULLION)}
		} else {
			sql += " AND 1 <> 1 "
		}
	} else {
		lockMeta := mt.Table.Contract.GetByNetIdName(int32(netId), jccommon.CONTRACT_NAME_NFTLock)
		if lockMeta != nil {
			sql = fmt.Sprintf(`
SELECT A.idx, A.net_id, A.token_type, A.token_id, A.contract_address, B.status FROM t_nft A LEFT JOIN t_order B
ON A.net_id = B.net_id AND A.contract_address = B.contract_address AND A.token_id = B.token_id
WHERE A.idx > %d AND A.net_id = %d AND (A.owner_address=? `,
				cursor, netId)
			sql += " OR (A.owner_address=? AND A.last_lock_address = ? AND A.token_type <> ?))"
			params = []string{
				accountAddress,
				strings.ToLower(lockMeta.GetAddress()),
				accountAddress,
				q5.ToString(jccommon.NFT_TYPE_GOLD_BULLION)}
		}
	}
	subFilters := []f5.DbQueryFilter{}
	{
		itemIds := map[int32]int32{}
		if searchName != "" {
			mt.Table.Item.Search(searchName, itemIds)
		}
		if len(itemIds) > 0 {
			inSub := `A.item_id IN (`
			i := 0
			for key, _ := range(itemIds) {
				if i == 0 {
					inSub += q5.ToString(key)
				} else {
					inSub += "," + q5.ToString(key)
				}
				i += 1
			}
			inSub += ")"
			q5.AppendSlice(&subFilters, f5.GetDbFilter().Custom(inSub).And())
		}
	}
	orderBy := ""
	rspObj := struct {
		ErrCode int32 `json:"errcode"`
		ErrMsg string `json:"errmsg"`
		Page common.StreamPagination `json:"page"`
		Rows []interface{} `json:"rows"`
	}{
		Rows : []interface{}{},
	}
	nfts := []*common.NftDto{}
	f5.GetGoStyleDb().StreamPageQuery(
		constant.BCNFT_DB,
		pageSize,
		cursor,
		sql,
		params,
		f5.GetDbFilter().Comp(subFilters...),
		orderBy,
		func (err error, pagination *f5.StreamPagination) {
			rspObj.Page.FillPage(pagination)
		},
		func (ds *f5.DataSet) {
			p := new(common.NftDto)
			p.NetId = q5.ToInt32(ds.GetByName("net_id"))
			p.ContractAddress = ds.GetByName("contract_address")
			p.TokenId = ds.GetByName("token_id")
			if ds.GetByName("status") == jccommon.ORDER_STATUS_ACTIVE {
				p.Param1 = 1
			} else {

			}
			q5.AppendSlice(&nfts, p)
		})
	GetCacheMgr().GetNfts(nfts)
	{
		for _, val := range nfts {
			jsonData := val.NftCache.GetJsonData()
			if jsonData != nil {
				if v, ok := jsonData.(map[string]interface{}); ok {
					v := q5.MapClone(v)
					if val.Param1 != 0 {
						v["on_sale"] = 1
					} else {
						v["on_sale"] = 0
					}
					jsonData = v
				}
			}
			q5.AppendSlice(&rspObj.Rows, jsonData)
		}
	}
	c.JSON(200, rspObj)
}

func (this *AssetApi) Listing(c *gin.Context) {
	pageSize := q5.AdjustRangeValue(q5.ToInt32(c.DefaultQuery("page_size", "")), 1, 20)
	cursor := q5.ToInt64(c.DefaultQuery("cursor", ""))
	netId := q5.ToInt64(c.Param("net_id"))
	accountAddress := strings.ToLower(c.Param("account_address"))
	searchName := c.DefaultQuery("search_name", "")
	sql := fmt.Sprintf(`
SELECT * FROM t_order WHERE net_id = %d AND status = "%s" AND seller_address=?`,
		netId,
		jccommon.ORDER_STATUS_ACTIVE)
	params := []string{accountAddress}
	subFilters := []f5.DbQueryFilter{}
	{
		itemIds := map[int32]int32{}
		if searchName != "" {
			mt.Table.Item.Search(searchName, itemIds)
		}
		if len(itemIds) > 0 {
			inSub := `item_id IN (`
			i := 0
			for key, _ := range(itemIds) {
				if i == 0 {
					inSub += q5.ToString(key)
				} else {
					inSub += "," + q5.ToString(key)
				}
				i += 1
			}
			inSub += ")"
			q5.AppendSlice(&subFilters, f5.GetDbFilter().Custom(inSub).And())
		}
	}
	orderBy := ""
	rspObj := struct {
		ErrCode int32 `json:"errcode"`
		ErrMsg string `json:"errmsg"`
		Page common.StreamPagination `json:"page"`
		Rows []struct{
			Event interface{} `json:"event"`
			Nft interface{} `json:"nft"`
			InShopCart int32 `json:"in_shopcart"`
			Price string `json:"price"`
		} `json:"rows"`
	}{
	}
	nfts := []*common.NftDto{}
	f5.GetGoStyleDb().StreamPageQuery(
		constant.BCNFT_DB,
		pageSize,
		cursor,
		sql,
		params,
		f5.GetDbFilter().Comp(subFilters...),
		orderBy,
		func (err error, pagination *f5.StreamPagination) {
			rspObj.Page.FillPage(pagination)
		},
		func (ds *f5.DataSet) {
			p := new(common.NftDto)
			p.NetId = q5.ToInt32(ds.GetByName("net_id"))
			p.ContractAddress = ds.GetByName("contract_address")
			p.TokenId = ds.GetByName("token_id")
			q5.DecodeJson(ds.GetByName("event_data"), &p.Payload)
			q5.AppendSlice(&nfts, p)
		})
	GetCacheMgr().GetNfts(nfts)
	{
		for _, val := range nfts {
			p := q5.NewSliceElement(&rspObj.Rows)
			p.Event = val.Payload
			p.Nft = val.NftCache.GetJsonData()
			/*
			if cartDb != nil && cartDb.GetGoods(val.NetId, val.ContractAddress, val.TokenId) != nil{
				p.InShopCart = 1
			}*/
		}
	}
	c.JSON(200, rspObj)
}
