package shopcart

import (
	"f5"
	"jccommon"
	"main/common"
	"main/constant"
	. "main/global"
	"q5"
	"strings"

	"github.com/gin-gonic/gin"
)

type ShopCartApi struct {
}

func (this *ShopCartApi) List(c *gin.Context) {
	openId := c.MustGet("open_id").(string)
	accountAddress := c.MustGet("account_address").(string)

	rspObj := &struct {
		ErrCode int32  `json:"errcode"`
		ErrMsg  string `json:"errmsg"`
		Data    []struct {
			Event interface{} `json:"event"`
			Nft   interface{} `json:"nft"`
		} `json:"data"`
	}{}
	q5.NewSlice(&rspObj.Data, 0, 10)
	f5.GetGoStyleDb().OrmSelectOne(
		constant.BCNFT_DB,
		"t_shopcart",
		[][]string{
			{"open_id", openId},
		},
		func(err error, ds *f5.DataSet) {
			if err != nil {
				rspObj.ErrCode = 500
				rspObj.ErrMsg = "server internal error"
				c.JSON(200, rspObj)
				return
			}
			nfts := []*common.NftDto{}
			if ds.Next() {
				cartDb := common.NewShopCart()
				if q5.DecodeJson(ds.GetByName("data"), cartDb) == nil {
					q5.Sort[common.ShopCartGoods](cartDb.Goods,
						func(a *common.ShopCartGoods, b *common.ShopCartGoods) bool {
							return a.SortIdx < b.SortIdx
						})
					dirty := false
					newGoods := []*common.ShopCartGoods{}
					priceType := ""
					priceContractAddress := ""
					for _, val := range cartDb.Goods {
						var err1 error
						wherekv := [][]string{
							{"net_id", q5.ToString(val.NetId)},
							{"contract_address", q5.ToString(val.ContractAddress)},
							{"token_id", q5.ToString(val.TokenId)},
							{"status", jccommon.ORDER_STATUS_ACTIVE},
						}

						if priceType != "" {
							wherekv = append(wherekv, []string{"src_price_item_type", priceType})
						}

						if priceContractAddress != "" {
							wherekv = append(wherekv, []string{"src_price_contract_address", priceContractAddress})
						}

						f5.GetGoStyleDb().OrmSelectOne(
							constant.BCNFT_DB,
							"t_order",
							wherekv,
							func(err error, ds *f5.DataSet) {
								err1 = err
								if err == nil {
									if ds.Next() {
										if ds.GetByName("seller_address") != accountAddress {
											val.PriceType = ds.GetByName("src_price_item_type")
											val.PriceContractAddress = ds.GetByName("src_price_contract_address")
											val.OrderId = ds.GetByName("order_id")

											if priceType == "" && priceContractAddress == "" {
												priceType = val.PriceType
												priceContractAddress = val.PriceContractAddress
											}

											p := new(common.NftDto)
											p.NetId = q5.ToInt32(ds.GetByName("net_id"))
											p.ContractAddress = ds.GetByName("contract_address")
											p.TokenId = ds.GetByName("token_id")
											p.Payload = map[string]interface{}{}
											q5.DecodeJson(ds.GetByName("event_data"), &p.Payload)
											q5.AppendSlice(&nfts, p)
											q5.AppendSlice(&newGoods, val)
										}
									} else {
										val.Flag = 1
									}
									dirty = true
								}
							})
						if err1 != nil {
							rspObj.ErrCode = 500
							rspObj.ErrMsg = "server internal error"
							c.JSON(200, rspObj)
							return
						}
					} //end for goods
					if dirty {
						cartDb.Goods = newGoods
						this.save(openId, cartDb)
					}
				}
			}
			GetCacheMgr().GetNfts(nfts)
			{
				for _, val := range nfts {
					p := q5.NewSliceElement(&rspObj.Data)
					p.Event = val.Payload
					p.Nft = val.NftCache.GetJsonData()
				}
			}
			c.JSON(200, rspObj)
		})
}

func (this *ShopCartApi) Add(c *gin.Context) {
	openId := c.MustGet("open_id").(string)
	accountAddress := c.MustGet("account_address").(string)

	rspObj := &struct {
		ErrCode int32  `json:"errcode"`
		ErrMsg  string `json:"errmsg"`
	}{}

	reqJson := struct {
		NetId  interface{} `json:"net_id"`
		Tokens []struct {
			TokenId         interface{} `json:"token_id"`
			ContractAddress string      `json:"contract_address"`
		} `json:"tokens"`
	}{}
	if err := c.ShouldBindJSON(&reqJson); err != nil {
		rspObj.ErrCode = 1
		rspObj.ErrMsg = "json parse error"
		c.JSON(200, rspObj)
		return
	}
	if len(reqJson.Tokens) > constant.SHOPCART_MAX_GOODS_NUM {
		rspObj.ErrCode = 2
		rspObj.ErrMsg = "token len error"
		c.JSON(200, rspObj)
		return
	}

	f5.GetGoStyleDb().OrmSelectOne(
		constant.BCNFT_DB,
		"t_shopcart",
		[][]string{
			{"open_id", openId},
		},
		func(err error, ds *f5.DataSet) {
			if err != nil {
				rspObj.ErrCode = 500
				rspObj.ErrMsg = "server internal error"
				c.JSON(200, rspObj)
				return
			}
			cartDb := common.NewShopCart()
			dirty := false
			if ds.Next() {
				if q5.DecodeJson(ds.GetByName("data"), cartDb) == nil {
				}
			}

			priceType := ""
			priceContractAddress := ""
			for _, g := range cartDb.Goods {
				inreq := false
				for _, req := range reqJson.Tokens {
					netId := q5.SafeToInt32(reqJson.NetId)
					contractAddress := req.ContractAddress
					tokenId := q5.SafeToString(req.TokenId)
					if g.NetId == netId && g.ContractAddress == contractAddress && g.TokenId == tokenId {
						inreq = true
						break
					}
				}

				if !inreq {
					priceType = g.PriceType
					priceContractAddress = g.PriceContractAddress
					break
				}
			}

			for _, val := range reqJson.Tokens {
				netId := q5.SafeToInt32(reqJson.NetId)
				contractAddress := val.ContractAddress
				tokenId := q5.SafeToString(val.TokenId)
				p := cartDb.GetGoods(netId, contractAddress, tokenId)
				if p != nil {
					var err1 error
					orderok := false
					f5.GetGoStyleDb().OrmSelectOne(
						constant.BCNFT_DB,
						"t_order",
						[][]string{
							{"net_id", q5.ToString(netId)},
							{"contract_address", contractAddress},
							{"token_id", tokenId},
							{"status", jccommon.ORDER_STATUS_ACTIVE},
						},
						func(err error, ds *f5.DataSet) {
							err1 = err
							if err != nil {
								return
							}
							if !ds.Next() {
								return
							}
							if ds.GetByName("seller_address") == accountAddress {
								return
							}

							p.OrderId = ds.GetByName("order_id")
							p.PriceType = ds.GetByName("src_price_item_type")
							p.PriceContractAddress = ds.GetByName("src_price_contract_address")
							if priceType == "" && priceContractAddress == "" {
								priceType = p.PriceType
								priceContractAddress = p.PriceContractAddress
								orderok = true
							} else if priceType != "" && priceContractAddress != "" &&
								priceType == p.PriceType && priceContractAddress == p.PriceContractAddress {
								orderok = true
							}
						})
					if err1 != nil {
						rspObj.ErrCode = 500
						rspObj.ErrMsg = "server internal error"
						c.JSON(200, rspObj)
						return
					}

					if orderok {
						cartDb.CurrSortIdx += 1
						p.SortIdx = cartDb.CurrSortIdx
						p.AddTime = q5.ToInt32(f5.GetApp().GetRealSeconds())
					} else {
						j := 0
						for _, g := range cartDb.Goods {
							if g != p {
								cartDb.Goods[j] = g
								j++
							}
						}
						cartDb.Goods = cartDb.Goods[:j]
					}
					dirty = true
				} else {
					var err1 error
					f5.GetGoStyleDb().OrmSelectOne(
						constant.BCNFT_DB,
						"t_order",
						[][]string{
							{"net_id", q5.ToString(netId)},
							{"contract_address", contractAddress},
							{"token_id", tokenId},
							{"status", jccommon.ORDER_STATUS_ACTIVE},
						},
						func(err error, ds *f5.DataSet) {
							err1 = err
							if err == nil {
								if ds.Next() {
									if ds.GetByName("seller_address") != accountAddress {
										cartDb.CurrSortIdx += 1
										g := common.NewShopCartGoods()
										g.NetId = netId
										g.ContractAddress = contractAddress
										g.TokenId = tokenId
										g.SortIdx = cartDb.CurrSortIdx
										g.AddTime = q5.ToInt32(f5.GetApp().GetRealSeconds())
										g.OrderId = ds.GetByName("order_id")
										g.PriceType = ds.GetByName("src_price_item_type")
										g.PriceContractAddress = ds.GetByName("src_price_contract_address")
										orderok := false
										if priceType == "" && priceContractAddress == "" {
											orderok = true
											priceType = g.PriceType
											priceContractAddress = g.PriceContractAddress
										} else if priceType != "" && priceContractAddress != "" && priceType == g.PriceType && priceContractAddress == g.PriceContractAddress {
											orderok = true
										}

										if orderok {
											q5.AppendSlice(&cartDb.Goods, g)
										}
									}
								} else {
								}
							}
						})
					if err1 != nil {
						rspObj.ErrCode = 500
						rspObj.ErrMsg = "server internal error"
						c.JSON(200, rspObj)
						return
					}
					dirty = true
				}
			}
			if dirty {
				cartDb.DeleteExcessData()
				this.save(openId, cartDb)
			}
			c.JSON(200, rspObj)
		})
}

func (this *ShopCartApi) Del(c *gin.Context) {
	openId := c.MustGet("open_id").(string)

	rspObj := &struct {
		ErrCode int32  `json:"errcode"`
		ErrMsg  string `json:"errmsg"`
	}{}

	reqJson := struct {
		NetId  interface{} `json:"net_id"`
		Tokens []struct {
			TokenId         interface{} `json:"token_id"`
			ContractAddress string      `json:"contract_address"`
		} `json:"tokens"`
	}{}
	if err := c.ShouldBindJSON(&reqJson); err != nil {
		rspObj.ErrCode = 1
		rspObj.ErrMsg = "json parse error"
		c.JSON(200, rspObj)
		return
	}

	f5.GetGoStyleDb().OrmSelectOne(
		constant.BCNFT_DB,
		"t_shopcart",
		[][]string{
			{"open_id", openId},
		},
		func(err error, ds *f5.DataSet) {
			if err != nil {
				rspObj.ErrCode = 500
				rspObj.ErrMsg = "server internal error"
				c.JSON(200, rspObj)
				return
			}
			cartDb := common.NewShopCart()
			dirty := false
			if ds.Next() {
				if q5.DecodeJson(ds.GetByName("data"), cartDb) == nil {
				}
			}
			newGoods := []*common.ShopCartGoods{}
			for _, val := range cartDb.Goods {
				found := false
				for _, val2 := range reqJson.Tokens {
					if val.NetId == q5.SafeToInt32(reqJson.NetId) &&
						q5.ToString(val.TokenId) == q5.SafeToString(val2.TokenId) &&
						strings.ToLower(val.ContractAddress) == strings.ToLower(val2.ContractAddress) {
						found = true
					}
				}
				if !found {
					q5.AppendSlice(&newGoods, val)
				} else {
					dirty = true
				}
			}
			cartDb.Goods = newGoods
			if dirty {
				this.save(openId, cartDb)
			}
			c.JSON(200, rspObj)
		})
}

func (this *ShopCartApi) Clear(c *gin.Context) {
	openId := c.MustGet("open_id").(string)

	rspObj := &struct {
		ErrCode int32  `json:"errcode"`
		ErrMsg  string `json:"errmsg"`
	}{}

	cartDb := common.NewShopCart()
	this.save(openId, cartDb)
	c.JSON(200, rspObj)
}

func (this *ShopCartApi) save(openId string, cartDb *common.ShopCart) {
	nowTime := f5.GetApp().GetRealSeconds()
	f5.GetGoStyleDb().Upsert(
		constant.BCNFT_DB,
		"t_shopcart",
		[][]string{
			{"open_id", openId},
		},
		[][]string{
			{"data", q5.EncodeJson(cartDb)},
			{"modifytime", q5.ToString(nowTime)},
		},
		[][]string{
			{"open_id", openId},
			{"data", q5.EncodeJson(cartDb)},
			{"createtime", q5.ToString(nowTime)},
			{"modifytime", q5.ToString(nowTime)},
		},
		func(err error, lastInsertId int64, rowsAffected int64) {

		})
}
