package vip

import (
	"f5"
	"main/constant"
	"main/mt"
	"main/service"
	"math/big"
	"q5"
	"strings"

	"github.com/gin-gonic/gin"
)

type VIPApi struct {
}

func (this *VIPApi) Info(c *gin.Context) {
	accountAddress := c.MustGet("account_address").(string)
	rspObj := struct {
		ErrCode           int32  `json:"errcode"`
		ErrMsg            string `json:"errmsg"`
		ContributionPoint string `json:"contribution_point"`
		Info              struct {
			BindPassport string `json:"bind_passport_address"`
			Email        string `json:"bind_email"`
		} `json:"info"`
	}{}

	{
		var dbErr error
		sql := "SELECT passport_address,lower_case_email FROM t_vip_bind WHERE account_address = ?;"
		params := []string{
			accountAddress,
		}
		f5.GetGoStyleDb().RawQuery(
			constant.BCNFT_DB,
			sql,
			params,
			func(err error, ds *f5.DataSet) {
				dbErr = err
				if err != nil {
					return
				}
				if ds.Next() {
					rspObj.Info.BindPassport = ds.GetByIndex(0)
					rspObj.Info.Email = ds.GetByIndex(1)
				}
			})

		if dbErr != nil {
			f5.RspErr(c, 500, "server internal error")
			return
		}
	}

	c.JSON(200, rspObj)
}

func (this *VIPApi) Bind(c *gin.Context) {
	accountAddress := c.GetString("account_address")
	if accountAddress == "" {
		f5.RspErr(c, 1, "bad request")
		return
	}
	reqObj := struct {
		Passport_jwt string `json:"passport_jwt"`
	}{}

	if err := c.ShouldBindJSON(&reqObj); err != nil {
		f5.RspErr(c, 1, err.Error())
		return
	}

	info := map[string]interface{}{}
	info["account_address"] = ""
	info["email"] = ""
	service.JwtAuth.JwtAuthHandle(reqObj.Passport_jwt, &info, constant.JWT_PASSPORT)
	passport_addr := info["account_address"].(string)
	if passport_addr == "" || accountAddress == passport_addr {
		f5.RspErr(c, 1, "bad request passport")
		return
	}

	email := info["email"].(string)

	bound := false
	{
		var dbErr error
		sql := "SELECT idx FROM t_vip_bind WHERE account_address = ? OR passport_address = ?;"
		params := []string{
			accountAddress,
			passport_addr,
		}
		f5.GetGoStyleDb().RawQuery(
			constant.BCNFT_DB,
			sql,
			params,
			func(err error, ds *f5.DataSet) {
				dbErr = err
				if err != nil {
					return
				}

				if ds.Next() {
					bound = true
				}
			})

		if dbErr != nil {
			f5.RspErr(c, 500, "server internal error")
			return
		}
	}

	if bound {
		f5.RspErr(c, 1, "account or passport bound")
		return
	}

	{
		ts := f5.GetApp().GetRealSeconds()
		var dbErr error
		f5.GetGoStyleDb().Insert(
			constant.BCNFT_DB,
			"t_vip_bind",
			[][]string{
				{"account_address", accountAddress},
				{"passport_address", passport_addr},
				{"email", email},
				{"lower_case_email", strings.ToLower(email)},
				{"createtime", q5.ToString(ts)},
				{"modifytime", q5.ToString(ts)},
			},
			func(err error, lastInsertId int64, rowsAffected int64) {
				if err != nil {
					dbErr = err
					return
				}
			})

		if dbErr != nil {
			f5.RspErr(c, 500, "server internal error")
			return
		}
	}

	f5.RspErr(c, 0, "")
}

func (this *VIPApi) Details(c *gin.Context) {
	accountAddress := c.GetString("account_address")
	if accountAddress == "" {
		f5.RspErr(c, 1, "bad request")
		return
	}

	rspObj := struct {
		ErrCode int32  `json:"errcode"`
		ErrMsg  string `json:"errmsg"`
		Info    struct {
			Level  int32 `json:"level"`
			Cur    int64 `json:"curpoint"`
			Target int64 `json:"target"`
		} `json:"info"`
	}{}

	{
		var dbErr error
		found := false
		sql := "SELECT * FROM t_vip_user WHERE account_address = ?"
		params := []string{
			accountAddress,
		}
		f5.GetGoStyleDb().RawQuery(
			constant.BCNFT_DB,
			sql,
			params,
			func(err error, ds *f5.DataSet) {
				dbErr = err
				if err != nil {
					return
				}

				if ds.Next() {
					found = true
					rspObj.Info.Level = q5.SafeToInt32(ds.GetByName("vip_lv"))
					cfg := mt.Table.StakingVip.GetById(q5.SafeToInt64(rspObj.Info.Level))
					if cfg != nil {
						rspObj.Info.Target = cfg.GetThisPoint()
						bigintobj := new(big.Int)
						escec_balance, ok := bigintobj.SetString(ds.GetByName("escec_balance"), 10)
						if !ok {
							f5.GetSysLog().Warning("escec_balance err:%s", ds.GetByName("escec_balance"))
						}

						escec_stacking, ok := bigintobj.SetString(ds.GetByName("escec_stacking"), 10)
						if !ok {
							f5.GetSysLog().Warning("escec_stacking err:%s", ds.GetByName("escec_stacking"))
						}

						curExec := bigintobj.Add(escec_balance, escec_stacking)

						escec_convert, ok := bigintobj.SetString(ds.GetByName("escec_convert"), 10)
						if !ok {
							f5.GetSysLog().Warning("escec_convert err:%s", ds.GetByName("escec_convert"))
						}

						last_convert_time := q5.SafeToInt64(ds.GetByName("last_convert_time"))
						nowsecs := f5.GetApp().GetRealSeconds()
						elaspse := bigintobj.SetInt64(nowsecs - last_convert_time)
						depositExec := bigintobj.Mul(escec_convert, elaspse)
						depositExec = bigintobj.Div(depositExec, bigintobj.SetInt64(365*24*3600))
						depositExec = bigintobj.Sub(escec_convert, depositExec)
						curExec = bigintobj.Add(curExec, depositExec)
						convercurExec := bigintobj.Div(curExec, bigintobj.SetInt64(10^18)).Int64()						
						nextlvcfg := mt.Table.StakingVip.GetById(q5.SafeToInt64(rspObj.Info.Level + 1))
						if nextlvcfg != nil && convercurExec >= cfg.GetRequire() {
							rspObj.Info.Cur = cfg.GetThisPoint() * (convercurExec - cfg.GetRequire()) /(nextlvcfg.GetRequire() - cfg.GetRequire())
						}
					}
				}
			})

		if dbErr != nil {
			f5.RspErr(c, 500, "server internal error")
			return
		}

		if !found {
			f5.RspErr(c, 1, "not vip")
			return
		}
	}

	c.JSON(200, rspObj)
}
