package mt

import (
	"f5"
	"fmt"
	"main/constant"
	"q5"
	"jccommon"
	"strings"
)

type Currency struct {
	currencyName    string
	contractName    string
	currencyDecimal int64
	contract        *Contract
}

type CurrencyTable struct {
	f5.CustomMetaTable
	netIdHash *q5.ConcurrentMap[int32, *q5.ConcurrentMap[string, *Currency]]
}

func (this *Currency) init(currencyName string, contractName string, currencyDecimal int64) {
	this.currencyName = currencyName
	this.contractName = contractName
	this.currencyDecimal = currencyDecimal
}

func (this *Currency) GetCurrencyName() string {
	return this.currencyName
}

func (this *Currency) GetContractName() string {
	return this.contractName
}

func (this *Currency) GetCurrencyDecimal() int64 {
	return this.currencyDecimal
}

func (this *Currency) GetContract() *Contract {
	return this.contract
}

func (this *Currency) check(netId int32) {
	if this.GetCurrencyDecimal() <= 0 {
		panic("Currency currency_decimal <= 0")
		return
	}
	if netId == 56 || netId == 97 {
		if this.GetCurrencyDecimal() != constant.BNB_RECHARGE_CURRENCY_MAX_DECIMAL {
			panic("Currency exchange_rate > uplimit")
			return
		}
	} else {
		if this.GetCurrencyDecimal() != constant.RECHARGE_CURRENCY_MAX_DECIMAL {
			panic("Currency exchange_rate > uplimit")
			return
		}
	}
}

func (this *CurrencyTable) GetByNetId(netId int32) *q5.ConcurrentMap[string, *Currency] {
	if v, ok := this.netIdHash.Load(netId); ok {
		return *v
	} else {
		return nil
	}
}

func (this *CurrencyTable) GetByNetIdAddress(netId int32, currencyAddress string) *Currency {
	currencysMeta := this.GetByNetId(netId)
	if currencysMeta == nil {
		return nil
	}
	var result *Currency
	currencyAddress = strings.ToLower(currencyAddress)
	currencysMeta.Range(
		func(key string, meta *Currency) bool {
			if meta.GetContract().GetAddress() == currencyAddress {
				result = meta
				return false
			}
			return true
		})
	return result
}

func (this *CurrencyTable) Load() {
	this.netIdHash = new(q5.ConcurrentMap[int32, *q5.ConcurrentMap[string, *Currency]])
	nets := []int32{}
	{
		if jsonStr, err := f5.ReadJsonFile("../config/nets.json"); err == nil {
			if err := q5.DecodeJson(jsonStr, &nets); err != nil {
				panic(fmt.Sprintf("load metafile json decode error %s %s", "nets.json", err))
			}
		} else {
			panic(fmt.Sprintf("load metafile error %s %s", "nets.json", err))
		}
	}
	{
		for _, val := range nets {
			netId := val
			fileName := fmt.Sprintf("../config/currencys/%d/currency.json", netId)
			if jsonStr, err := f5.ReadJsonFile(fileName); err == nil {
				f5.GetSysLog().Info("load currency %s", fileName)
				currencysCfg := []struct {
					CurrencyName    string `json:"currency_name"`
					ContractName    string `json:"contract_name"`
					CurrencyDecimal int64  `json:"currency_decimal"`
				}{}
				if err := q5.DecodeJson(jsonStr, &currencysCfg); err != nil {
					panic(fmt.Sprintf("load metafile json decode error %s %s", "currency.json", err))
				}
				for _, currencyCfg := range currencysCfg {
					p := new(Currency)
					p.init(currencyCfg.CurrencyName, currencyCfg.ContractName, currencyCfg.CurrencyDecimal)
					p.check(netId)
					currencysMeta := this.GetByNetId(netId)
					if currencysMeta == nil {
						currencysMeta = new(q5.ConcurrentMap[string, *Currency])
						this.netIdHash.Store(netId, currencysMeta)
					}
					currencysMeta.Store(p.GetCurrencyName(), p)
				}
			}
		}
	}
}

func (this *CurrencyTable) PostInit1() {
	this.netIdHash.Range(
		func(key int32, val *q5.ConcurrentMap[string, *Currency]) bool {
			netId := key
			val.Range(
				func(key2 string, val2 *Currency) bool {
					currencyMeta := val2
					Table.Recharge.Traverse(func(ele *Recharge) bool {
						if int64(ele.GetPrice())*constant.RECHARGE_CURRENCY_MAX_EXCHANGE_RAET !=
							int64(ele.GetDiamond()) {
							panic(fmt.Sprintf("Currency verifyerror net_id:%d id:%d", netId, ele.GetId()))
						}
						return true
					})
					contractMeta := Table.Contract.GetByNetIdName(netId, currencyMeta.GetContractName())
					if contractMeta == nil {
						panic(fmt.Sprintf("currency contract not found"))
					}
					this.verifyCurrencyAddress(netId, currencyMeta.GetContractName(), contractMeta.GetAddress())
					currencyMeta.contract = contractMeta
					return true
				})
			return true
		})
}


func (this *CurrencyTable) verifyCurrencyAddress(netId int32, name string, address string) {
	switch netId {
	case jccommon.ETH_NET_ID:
		{
			if name == "BEUSDC" {
				if (strings.ToLower(address) != strings.ToLower(jccommon.ETH_USDC_ADDRESS)) {
					panic(fmt.Sprintf("verifyCurrencyAddress error net:%d name:%s address:%s",
						netId, name, address))
				}
			} else if name == "BEUSDT" {
				if (strings.ToLower(address) != strings.ToLower(jccommon.ETH_USDT_ADDRESS)) {
					panic(fmt.Sprintf("verifyCurrencyAddress error net:%d name:%s address:%s",
						netId, name, address))
				}
			} else {
				panic(fmt.Sprintf("verifyCurrencyAddress error net:%d name:%s address:%s",
					netId, name, address))
			}
		}
	case jccommon.IMTBL_NET_ID:
		{
			if name == "BEUSDC" {
				if (strings.ToLower(address) != strings.ToLower(jccommon.IMTBL_USDC_ADDRESS)) {
					panic(fmt.Sprintf("verifyCurrencyAddress error net:%d name:%s address:%s",
						netId, name, address))
				}
			} else {
				panic(fmt.Sprintf("verifyCurrencyAddress error net:%d name:%s address:%s",
					netId, name, address))
			}
		}
	case jccommon.ARB_NET_ID:
		{
			if name == "BEUSDC" {
				if (strings.ToLower(address) != strings.ToLower(jccommon.ARB_USDC_ADDRESS)) {
					panic(fmt.Sprintf("verifyCurrencyAddress error net:%d name:%s address:%s",
						netId, name, address))
				}
			} else if name == "BEUSDT" {
				if (strings.ToLower(address) != strings.ToLower(jccommon.ARB_USDT_ADDRESS)) {
					panic(fmt.Sprintf("verifyCurrencyAddress error net:%d name:%s address:%s",
						netId, name, address))
				}
			} else {
				panic(fmt.Sprintf("verifyCurrencyAddress error net:%d name:%s address:%s",
					netId, name, address))
			}
		}
	case jccommon.BNB_NET_ID:
		{
			if name == "BEUSDC" {
				if (strings.ToLower(address) != strings.ToLower(jccommon.BNB_USDC_ADDRESS)) {
					panic(fmt.Sprintf("verifyCurrencyAddress error net:%d name:%s address:%s",
						netId, name, address))
				}
			} else if name == "BEUSDT" {
				if (strings.ToLower(address) != strings.ToLower(jccommon.BNB_USDT_ADDRESS)) {
					panic(fmt.Sprintf("verifyCurrencyAddress error net:%d name:%s address:%s",
						netId, name, address))
				}
			} else {
				panic(fmt.Sprintf("verifyCurrencyAddress error net:%d name:%s address:%s",
					netId, name, address))
			}
		}
	}
}
