package service

import (
	"f5"
	"fmt"
	"main/constant"
	"q5"
	"time"
)

type accountContribution struct {
	history          float64
	stakingContri    float64
	gamecontribution float64
	rechargeContri   float64
	gcTime           int64
	loweremail       string
	accountid        string
}

func (ac *accountContribution) sum() float64 {
	return ac.history + ac.stakingContri + ac.gamecontribution + ac.rechargeContri
}

type contribution struct {
	historyContribution  float64
	stakingContribution  float64
	rechargeContribution float64
	gameContribution     float64
	gcTime               int64

	accountContributionlist q5.ConcurrentMap[string, *accountContribution]
	emailContributionlist   q5.ConcurrentMap[string, *accountContribution]
}

func (this *contribution) init() {
	this.accountContributionlist = q5.ConcurrentMap[string, *accountContribution]{}
	this.emailContributionlist = q5.ConcurrentMap[string, *accountContribution]{}
	this.GetGlobalContribution(true)
	go this.checkContributionList()
}

func (this *contribution) unInit() {

}

func (c *contribution) sum() float64 {
	return c.historyContribution + c.stakingContribution + c.rechargeContribution + c.gameContribution
}

func (this *contribution) GetEmailContributionAccountid(email string) (float64, string, error) {
	info, exist := this.emailContributionlist.Load(email)
	if exist {
		return (*info).sum(), (*info).accountid, nil
	}

	accountid := ""
	accountAddress := ""
	sql := `SELECT account_id, address FROM t_immutable_account WHERE idx > 0 AND lower_case_email = ?`
	f5.GetGoStyleDb().RawQuery(
		constant.ACCOUNT_DB,
		sql,
		[]string{email},
		func(err error, ds *f5.DataSet) {
			if err != nil {
				return
			}

			if ds.Next() {
				accountid = ds.GetByIndex(0)
				accountAddress = ds.GetByIndex(1)
			}
		})

	if accountAddress != "" {
		contrinfo := new(accountContribution)
		contrinfo.loweremail = email
		contrinfo.accountid = accountid
		this.accountContributionlist.Store(accountAddress, contrinfo)
		contri, _ := this.GetAddressContribution(accountAddress, false)

		return contri, accountid, nil
	}

	return 0, accountid, nil
}

func (this *contribution) GetEmailAccountId(accountAddress string) (string, string) {
	accinfo, exist := this.accountContributionlist.Load(accountAddress)
	if exist {
		return (*accinfo).loweremail, (*accinfo).accountid
	}

	return "", ""
}

func (this *contribution) GetAddressContribution(accountAddress string, onlyrecharge bool) (float64, error) {
	if accountAddress == "" {
		return 0, fmt.Errorf("empty account")
	}
	accinfo, exist := this.accountContributionlist.Load(accountAddress)
	var beforcontribution float64 = 0
	if exist {
		beforcontribution = (*accinfo).sum()
	}

	if nowseconds := f5.GetApp().GetRealSeconds(); !exist || (*accinfo).gcTime+60 < nowseconds {
		if !exist {
			info := new(accountContribution)
			this.accountContributionlist.Store(accountAddress, info)
			accinfo = &info
		}
		beforcontribution = (*accinfo).sum()

		if (*accinfo).loweremail == "" {
			sql := `SELECT account_id FROM t_user WHERE idx > 0 AND address = ?`
			f5.GetGoStyleDb().RawQuery(
				constant.GAME_DB,
				sql,
				[]string{accountAddress},
				func(err error, ds *f5.DataSet) {
					if err != nil {
						return
					}

					if ds.Next() {
						(*accinfo).accountid = ds.GetByIndex(0)
					}
				})

			if (*accinfo).accountid != "" {
				sql := `SELECT lower_case_email FROM t_immutable_account WHERE idx > 0 AND account_id = ?`
				f5.GetGoStyleDb().RawQuery(
					constant.ACCOUNT_DB,
					sql,
					[]string{(*accinfo).accountid},
					func(err error, ds *f5.DataSet) {
						if err != nil {
							return
						}

						if ds.Next() {
							(*accinfo).loweremail = ds.GetByIndex(0)
						}
					})
			}

			if (*accinfo).loweremail != "" {
				this.emailContributionlist.Store((*accinfo).loweremail, *accinfo)
			}
		}

		{
			sql := `SELECT contribution FROM t_contribution WHERE idx > 0 and account_address = ?`
			f5.GetGoStyleDb().RawQuery(
				constant.BCNFT_DB,
				sql,
				[]string{accountAddress},
				func(err error, ds *f5.DataSet) {
					if err != nil {
						return
					}

					if ds.Next() {
						(*accinfo).history, _ = q5.ToFloat64Ex(ds.GetByIndex(0))
					}
				})

			sql = `SELECT SUM(contribution) FROM t_staking_daily_settlement WHERE idx > 0 and account_address = ?`
			f5.GetGoStyleDb().RawQuery(
				constant.BCNFT_DB,
				sql,
				[]string{accountAddress},
				func(err error, ds *f5.DataSet) {
					if err != nil {
						return
					}

					if ds.Next() {
						(*accinfo).stakingContri, _ = q5.ToFloat64Ex(ds.GetByIndex(0))
					}
				})
		}

		if (*accinfo).accountid != "" {
			sql := `SELECT contribution FROM t_contribution WHERE idx > 0 AND account_id = ?`
			f5.GetGoStyleDb().RawQuery(
				constant.GAME_DB,
				sql,
				[]string{(*accinfo).accountid},
				func(err error, ds *f5.DataSet) {
					if err != nil {
						return
					}

					if ds.Next() {
						(*accinfo).gamecontribution, _ = q5.ToFloat64Ex(ds.GetByIndex(0))
					}
				})
		}

		{
			params := []string{accountAddress}
			sql := `SELECT SUM(return_contribution) FROM t_recharge_return_contribution WHERE idx > 0 AND user_identity IN (?`
			if (*accinfo).loweremail != "" {
				sql += ", ?"
				params = append(params, (*accinfo).loweremail)
			}
			sql += ")"

			f5.GetGoStyleDb().RawQuery(
				constant.BCNFT_DB,
				sql,
				params,
				func(err error, ds *f5.DataSet) {
					if err != nil {
						return
					}

					if ds.Next() {
						(*accinfo).rechargeContri, _ = q5.ToFloat64Ex(ds.GetByIndex(0))
					}
				})
		}

		(*accinfo).gcTime = nowseconds
	}

	if beforcontribution > 0.000001 && (*accinfo).sum() > beforcontribution+0.000001 {
		this.GetGlobalContribution(true)
	}

	if onlyrecharge {
		return (*accinfo).rechargeContri, nil
	}

	return (*accinfo).sum(), nil
}

func (this *contribution) GetGlobalContribution(instant bool) (float64, error) {
	if this.historyContribution < 0.000001 {
		sql := `SELECT SUM(contribution) FROM t_contribution WHERE idx > 0`
		f5.GetGoStyleDb().RawQuery(
			constant.BCNFT_DB,
			sql,
			[]string{},
			func(err error, ds *f5.DataSet) {
				if err != nil {
					return
				}

				if ds.Next() {
					this.historyContribution, _ = q5.ToFloat64Ex(ds.GetByIndex(0))
				}
			})
		}

	if nowseconds := f5.GetApp().GetRealSeconds(); nowseconds > this.gcTime+60 || instant {
		sql := `SELECT SUM(contribution) FROM t_staking_daily_settlement WHERE idx > 0`
		f5.GetGoStyleDb().RawQuery(
			constant.BCNFT_DB,
			sql,
			[]string{},
			func(err error, ds *f5.DataSet) {
				if err != nil {
					return
				}

				if ds.Next() {
					this.stakingContribution, _ = q5.ToFloat64Ex(ds.GetByIndex(0))
				}
			})

		sql = `SELECT SUM(contribution) FROM t_contribution WHERE idx > 0`
		f5.GetGoStyleDb().RawQuery(
			constant.GAME_DB,
			sql,
			[]string{},
			func(err error, ds *f5.DataSet) {
				if err != nil {
					return
				}

				if ds.Next() {
					this.gameContribution, _ = q5.ToFloat64Ex(ds.GetByIndex(0))
				}
			})
		sql = `SELECT SUM(return_contribution) FROM t_recharge_return_contribution WHERE idx > 0`
		f5.GetGoStyleDb().RawQuery(
			constant.BCNFT_DB,
			sql,
			[]string{},
			func(err error, ds *f5.DataSet) {
				if err != nil {
					return
				}

				if ds.Next() {
					this.rechargeContribution, _ = q5.ToFloat64Ex(ds.GetByIndex(0))
				}
			})

		this.gcTime = nowseconds
	}

	return this.sum(), nil
}

func (this *contribution) checkContributionList() {
	fmt.Println("checkContributionList start")
	for {
		if time.Now().UTC().Hour() == 0 && time.Now().UTC().Minute() == 0 {
			nowseconds := f5.GetApp().GetRealSeconds()
			deletelist := []string{}
			this.accountContributionlist.Range(func(key string, value *accountContribution) bool {
				if value.gcTime+86400 < nowseconds {
					deletelist = append(deletelist, key)
				}
				return true
			})

			for _, account := range deletelist {
				v, _ := this.accountContributionlist.Load(account)
				if (*v).loweremail != "" {
					this.emailContributionlist.Delete((*v).loweremail)
				}
				this.accountContributionlist.Delete(account)
			}
			f5.GetSysLog().Info("delete contribution address cache count:%d", len(deletelist))
		}

		f5.GetSysLog().Info("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<")
		f5.GetSysLog().Info("contribution cache total:%.2f, recharge:%.2f, ingame:%.2f, history:%.2f",
			this.sum(),
			this.rechargeContribution,
			this.gameContribution,
			this.historyContribution)
		f5.GetSysLog().Info(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>")
		time.Sleep((time.Second * 60))
	}
}
