package service

import (
	"f5"
	"fmt"
	"jccommon"
	"main/mt"
	"marketserver/constant"
	"q5"
	"strings"
)

type jwtauth struct {
}

func (this *jwtauth) init() {
}

func (this *jwtauth) unInit() {
}

func (this *jwtauth) JwtAuthHandle(srcToken string, rspinfo *map[string]interface{}, jwtType int32) {
	jwtToken := ""
	if len(srcToken) > 8 {
		jwtToken = srcToken[7:]
	}

	tmpStrings := q5.StrSplit(jwtToken, ".")
	if len(tmpStrings) > 3 {
		if jwtType == constant.JWT_METAMAST {
			this.internalMetaMaskJwtAuth(jwtToken, rspinfo)
		}
	} else {
		if jwtType == constant.JWT_PASSPORT {
			this.internalImmutJwtAuth(jwtToken, rspinfo)
		}
	}
}

func (this *jwtauth) internalImmutJwtAuth(jwtToken string, rspinfo *map[string]interface{}) {
	params := map[string]string{
		"c": "Jwt",
		"a": "verify",
	}
	jsonReqObj := &struct {
		JwksUri string `json:"jwksUri"`
		Data    string `json:"data"`
	}{
		JwksUri: mt.Table.Config.GetJwksUri(),
		Data:    jwtToken,
	}
	jsonRspObj := &struct {
		ErrCode interface{} `json:"errcode"`
		ErrMsg  string      `json:"errmsg"`
		Decoded struct {
			Sub           string `json:"sub"`
			Email         string `json:"email"`
			EmailVerified bool   `json:"email_verified"`
			Sid           string `json:"sid"`
			Passport      struct {
				ZkevmEthAddress string `json:"zkevm_eth_address"`
			} `json:"passport"`
		} `json:"decoded"`
	}{}
	rspObj := &struct {
		ErrCode interface{} `json:"errcode"`
		ErrMsg  string      `json:"errmsg"`
	}{}
	paramsStr := q5.EncodeJson(jsonReqObj)
	url := fmt.Sprintf("%s/webapp/index.php", mt.Table.Web3ServiceCluster.RandElement().GetUrl())
	f5.GetHttpCliMgr().SendGoStyleJsonRspPost(
		url,
		params,
		jsonRspObj,
		q5.HTTP_HEADER_JSON,
		paramsStr,
		func(rsp f5.HttpCliResponse) {
			if rsp.GetErr() != nil ||
				!rsp.JsonParseOk() {
				return
			}
			rspObj.ErrCode = q5.SafeToInt32(jsonRspObj.ErrCode)
			rspObj.ErrMsg = jsonRspObj.ErrMsg
			if q5.SafeToInt32(rspObj.ErrCode) != 0 {
				return
			}
			openId := fmt.Sprintf("%d_2006_%s", jccommon.IMTBL_CHANNEL, jsonRspObj.Decoded.Sub)
			(*rspinfo)["open_id"] = openId
			(*rspinfo)["account_address"] = jsonRspObj.Decoded.Passport.ZkevmEthAddress
			(*rspinfo)["email"] = jsonRspObj.Decoded.Email
		})
}

func (this *jwtauth) internalMetaMaskJwtAuth(jwtToken string, rspinfo *map[string]interface{}) {
	params := map[string]string{
		"c": "MetaMaskJwt",
		"a": "verify",
	}
	jsonReqObj := &struct {
		Data string `json:"data"`
	}{
		Data: jwtToken,
	}
	jsonRspObj := &struct {
		ErrCode interface{} `json:"errcode"`
		ErrMsg  string      `json:"errmsg"`
		Decoded struct {
			Id      string `json:"id"`
			OpenId  string `json:"openid"`
			Plat    int32  `json:"plat"`
			Version int32  `json:"version"`
			Email   string `json:"email"`
		} `json:"decoded"`
	}{}
	rspObj := &struct {
		ErrCode interface{} `json:"errcode"`
		ErrMsg  string      `json:"errmsg"`
	}{}
	paramsStr := q5.EncodeJson(jsonReqObj)
	url := fmt.Sprintf("%s/webapp/index.php", mt.Table.Web3ServiceCluster.RandElement().GetUrl())
	f5.GetHttpCliMgr().SendGoStyleJsonRspPost(
		url,
		params,
		jsonRspObj,
		q5.HTTP_HEADER_JSON,
		paramsStr,
		func(rsp f5.HttpCliResponse) {
			if rsp.GetErr() != nil ||
				!rsp.JsonParseOk() {
				return
			}
			rspObj.ErrCode = q5.SafeToInt32(jsonRspObj.ErrCode)
			rspObj.ErrMsg = jsonRspObj.ErrMsg
			if q5.SafeToInt32(rspObj.ErrCode) != 0 {

				return
			}
			if jsonRspObj.Decoded.Plat != jccommon.BC_POLY_POLY_METAKASK {
				f5.GetSysLog().Info("not support this platform:%s", rsp.GetRawData())
				return
			}
			openId := fmt.Sprintf("%d_2006_%s",
				jccommon.BC_CHANNEL,
				strings.ToLower(jsonRspObj.Decoded.OpenId))
			(*rspinfo)["open_id"] = openId
			(*rspinfo)["account_address"] = strings.ToLower(jsonRspObj.Decoded.OpenId)
			(*rspinfo)["email"] = strings.ToLower(jsonRspObj.Decoded.Email)
		})
}
