package common

import (
	"net"

	"cs"
	"f5"
	"ss"

	proto "github.com/golang/protobuf/proto"
)

type BattleItem struct {
	ItemId int32 `json:"item_id"`
	ItemNum int32 `json:"item_num"`
	ItemType int32 `json:"item_type"`
	ItemSubType int32 `json:"item_sub_type"`
}

type UserInfo struct {
	//Activated   string `json:"activated"`
	//RenameCount string `json:"rename_count"`
	AccountId   string `json:"account_id"`
	Name        string `json:"name"`
	HeadId      string `json:"head_id"`
	//HeroId      string `json:"hero_id"`
	HeadFrame   string `json:"head_frame"`
	TotalLucky   interface{} `json:"total_lucky"`
	AdmissionItemNum   interface{} `json:"admission_item_num"`
	CircuitScore   interface{} `json:"circuit_score"`
	VipLv interface{} `json:"vip_lv"`
	VipExp interface{} `json:"vip_exp"`
	HasVipLucky interface{} `json:"has_vip_lucky"`
	HasLuckySymbol interface{} `json:"has_lucky_symbol"`
}

type HeroInfo struct {
	HeroUniId string `json:"hero_uniid"`
	HeroId string `json:"hero_id"`
	Quality string `json:"quality"`
	SpecSkill   string `json:"spec_skill"`
	Wealth   interface{} `json:"wealth"`
	ValidLefttime   interface{} `json:"valid_lefttime"`
	CurrentTimes   interface{} `json:"current_times"`
	TotalTimes   interface{} `json:"total_times"`
	SkinId   interface{} `json:"skin_id"`
}

type LoginRsp struct {
	Errcode int    `json:"errcode"`
	Errmsg  string `json:"errmsg"`
	Info struct {
		UserInfo UserInfo `json:"user_info"`
		HeroInfo HeroInfo `json:"hero_info"`
		MapInfo *MapInfoRsp  `json:"map_info"`
	} `json:"info"`
}

type UserRsp struct {
	Errcode int    `json:"errcode"`
	Errmsg  string `json:"errmsg"`
	Info struct {
		UserInfo UserInfo `json:"user_info"`
		HeroInfo HeroInfo `json:"hero_info"`
	} `json:"info"`
}

type MapInfoRsp struct {
	ModeId interface{} `json:"mode_id"`
	MapId int32 `json:"map_id"`
	IsMoba int32 `json:"is_moba"`
	IsOpen int32 `json:"is_open"`
	//NeedTicket int32 `json:"need_ticket"`
}

type UserQueryStatusRsp struct {
	Errcode int    `json:"errcode"`
	Errmsg  string `json:"errmsg"`
	Users []struct {
		AccountId string `json:"account_id"`
		Battling int32 `json:"battling"`
		Online int32 `json:"online"`
	}
}

type UpdateBattleInfoRsp struct {
	Errcode int    `json:"errcode"`
	Errmsg  string `json:"errmsg"`
	Info struct {
		UserInfo UserInfo `json:"user_info"`
		HeroInfo HeroInfo `json:"hero_info"`
	} `json:"info"`
}

type Team interface {
	GetTeamUuid() string
	CanJoin(string) bool
	Join(Player) bool
	ReJoin(Player)
	GetMemberByAccountId(string) Player
	IsCopy() bool
	GetCopyIdx() int32
	SendUpdateNotify()
	SendStateNotify()
	IsLeader(Player) bool
	Started() bool
	Matching() bool
	CanStartGame(Player) bool
	StartGame()
	GetMemberNum() int32
	IsLock() bool
	NeedMatchRealPlayer() bool
	HasOnlinePlayer() bool

	Leave(Player)
	Disband()
	KickOut(Player, string)
	HandoverLeader(Player, string)
	CancelMatch()
	SetMapInfo(*MapInfoRsp)
	GetMapId() int32
	GetModeId() int32
	IsMobaMode() bool
}

type TeamMgr interface {
	CreateTeam(Player, *cs.CMLogin, *MapInfoRsp) Team
	GetTeamByUuid(string) Team
	OnEnterNewUser(Player)
}

type Player interface {
	GetSocket() f5.WspCliConn
	SendMsg(proto.Message)
	GetAccountId() string
	GetSessionId() string
	GetZoneId() int32
	GetNodeId() int32
	IsOnline() bool
	GetTeam() Team
	SetTeam(Team)
	IsReady() bool
	GetSpecSkill() int32
	GetHeroUniid() string
	GetPermission() int32
	SetPermission(int32)
	FillMFTeamMember(*cs.MFTeamMember)
	GetSortIdx() int32
	SetSortIdx(int32)
	IsBattling() bool
	SetBattling(bool)
	GenNextCopy() Player
	GetBattleItems() []*BattleItem
}

type PlayerMgr interface {
	ProcessCMMsg(*cs.CsNetMsgHandler, *f5.MsgHdr)
	GetPlayerBySocket(f5.WspCliConn) Player
	OnSocketClose(f5.WspCliConn)
}

type MatchMgr interface {
}

type HistoryMgr interface {
	AddBattle(string, int32, int32)
	PushLastBattleInfo(Player)
	CanStart(string, int32, int32) bool
}

type WspListener interface {
	ProcessSSMMsg(*ss.SsNetMsgHandler, *f5.MsgHdr)
	SendProxyMsg(net.Conn, uint16, proto.Message)
}

type HttpListener interface {
}

type App interface {
	Run(func(), func())
	AddNetMsg(*f5.MsgHdr)
}
