package history

import (
	"q5"
	"f5"
	"cs"
	"jccommon"
	"main/common"
	"main/mt"
	"github.com/golang/protobuf/proto"
)

type historyMgr struct {
	userHash map[string]*user
}

func (this *historyMgr) Init() {
	this.userHash = make(map[string]*user)
}

func (this *historyMgr) UnInit() {
}

func (this *historyMgr) AddBattle(accountId string, mapId int32, modeId int32) {
	u := this.getUser(accountId)
	if u == nil {
		u = new(user)
		u.init()
		u.accountId = accountId
		this.userHash[accountId] = u
	}
	u.addBattle(mapId, modeId)
}

func (this *historyMgr) PushLastBattleInfo(hum common.Player) {
	u := this.getUser(hum.GetAccountId())
	if u == nil {
		return
	}
	msg := new(cs.SMPushLastBattleInfo)
	q5.NewSlice(&msg.BattleList, 0, 10)
	for key, val := range(u.mapAndModeHash) {
		mapId := int32(key)
		modeId := int32(key >> 32)
		p := new(cs.MFLastBattleInfo)
		p.MapId = proto.Int32(mapId)
		p.ModeId = proto.Int32(modeId)
		p.StartTime = proto.Int32(val)
		if modeId == jccommon.MAP_MODE_CIRCUIT {
			p.IntervalTime = proto.Int32(mt.Table.Config.Get().GetCircuitModeIntervalTime())
		}
		q5.AppendSlice(&msg.BattleList, p)
	}
	hum.SendMsg(msg)
}

func (this *historyMgr) CanStart(accountId string, mapId int32, modeId int32) bool {
	u := this.getUser(accountId)
	if u == nil && modeId == jccommon.MAP_MODE_CIRCUIT {
		key := q5.MkInt64(mapId, modeId)
		if startTime, ok := u.mapAndModeHash[key]; ok {
			nowTime := q5.ToInt32(f5.GetApp().GetRealSeconds())
			if nowTime - startTime < mt.Table.Config.Get().GetCircuitModeIntervalTime() {
				return false
			}
		}
	}
	return true
}

func (this *historyMgr) getUser(accountId string) *user {
	if u, ok := this.userHash[accountId]; ok {
		return u
	} else {
		return nil
	}
}


func (this *historyMgr) removeUser(accountId string) {
	delete(this.userHash, accountId)
}
