package listener

import (
	"fmt"
	"net"

	"cs"
	"f5"
	"main/mt"
	"q5"
	"ss"

	proto "github.com/golang/protobuf/proto"
	"main/constant"
	. "main/global"
)

type WSPListener struct {
	ss.MsgHandlerImpl
	listener net.Listener
	ch       chan *f5.MsgHdr
	msgList  q5.ListHead
}

func (this *WSPListener) Init() {
	this.ch = make(chan *f5.MsgHdr)
	listener, err := net.Listen("tcp", "0.0.0.0:"+
		q5.ToString(mt.Table.MatchCluster.GetListenPort()))
	if err != nil {
		panic(err)
	} else {
		this.listener = listener
		go this.parseNetPkt()
		go this.accept()
	}
}

func (this *WSPListener) UnInit() {
}

func (this *WSPListener) SS_ping(hdr *f5.MsgHdr, msg *ss.SS_Ping) {
	rspMsg := ss.SS_Pong{}
	this.SendProxyMsg(hdr.Conn, 0, &rspMsg)
}

func (this *WSPListener) SS_WSP_SocketDisconnect(hdr *f5.MsgHdr, msg *ss.SS_WSP_SocketDisconnect) {
	GetPlayerMgr().OnSocketClose(hdr.GetSocket())
}

func (this *WSPListener) accept() {
	fmt.Println("accpet")
	for {
		conn, err := this.listener.Accept()
		if err == nil {
			fmt.Println("connected")
			go this.socketRead(conn)
		}
	}
}

func (this *WSPListener) socketRead(conn net.Conn) {
	buf := make([]byte, 1024*64)
	recvBufLen := 0
	recvBuf := make([]byte, 1024*64*2)
	for {
		bufLen, err := conn.Read(buf)
		if err != nil {
			f5.GetApp().AddIMMsg(constant.IM_WSP_CLOSE, []interface{} {
				conn,
			})
			return
		}
		if bufLen > 0 {
			alreadyReadBytes := 0
			for {
				{
					readBytes := bufLen - alreadyReadBytes
					if readBytes > constant.MAX_PACKET_LEN-recvBufLen {
						readBytes = constant.MAX_PACKET_LEN - recvBufLen
					}

					if readBytes > 0 {
						copy(recvBuf[recvBufLen:],
							buf[alreadyReadBytes:alreadyReadBytes+readBytes])
						recvBufLen += readBytes
						alreadyReadBytes += readBytes
					}
				}

				offset := 0
				prevOffset := 0
				for {
					prevOffset = offset
					this.decodePacket(conn, recvBuf, &offset, recvBufLen)
					if prevOffset >= offset || offset >= recvBufLen {
						break
					}
				}

				if offset > 0 && offset < recvBufLen {
					copy(recvBuf[:], recvBuf[offset:])
				}
				recvBufLen -= offset
				if recvBufLen >= constant.MAX_PACKET_LEN {
					panic("recv max packet")
				}

				if alreadyReadBytes >= bufLen {
					break
				}
			}
		}
	}
}

func (this *WSPListener) decodePacket(conn net.Conn, buf []byte, offset *int, bufLen int) {
	warning := false
	for bufLen-*offset >= f5.WSPROXYPACKHEAD_C_SIZE {
		msgHead := f5.WSProxyPackHead_C{}
		msgHead.Read(buf, *offset)
		if msgHead.MagicCode == f5.NET_MSG_MAGIC_CODE {
			if bufLen-*offset < int(f5.WSPROXYPACKHEAD_C_SIZE)+int(msgHead.PackLen) {
				continue
			}
			hdr := new(f5.MsgHdr)
			hdr.Conn = conn
			hdr.MsgId = msgHead.MsgId
			hdr.SocketHandle = msgHead.SocketHandle
			hdr.SeqId = msgHead.SeqId
			hdr.Data = buf[int(*offset)+f5.WSPROXYPACKHEAD_C_SIZE : int(*offset)+f5.WSPROXYPACKHEAD_C_SIZE+int(msgHead.PackLen)]
			this.ch <- hdr
			*offset += f5.WSPROXYPACKHEAD_C_SIZE + int(msgHead.PackLen)
		} else {
			warning = true
			*offset++
			continue
		}
	}

	if warning {
		f5.GetSysLog().Warning("收到client非法数据包")
	}
}

func (this *WSPListener) parseNetPkt() {
	for {
		if hdr, ok := <-this.ch; ok {
			if hdr.MsgId < f5.WSP_SS_MAX_MSGID {
				hdr.Msg = ss.ParsePb(hdr.MsgId, hdr.Data)
				if hdr.Msg != nil {
					hdr.Entry.Init(hdr)
					GetApp().AddNetMsg(hdr)
				}
			} else {
				hdr.Msg = cs.ParsePb(hdr.MsgId, hdr.Data)
				if hdr.Msg != nil {
					hdr.Entry.Init(hdr)
					GetApp().AddNetMsg(hdr)
				}
			}
		}
	}
}

func (this *WSPListener) SendProxyMsg(conn net.Conn, socketHandle uint16, msg proto.Message) {
	netMsg := msg.(f5.NetMsg)
	if !f5.IsOnlineEnv() {
		f5.GetSysLog().Info("%s %s", q5.GetTypeName(msg), q5.EncodeJson(msg))
	}
	msgData, err := proto.Marshal(msg)
	if err != nil {
		return
	}
	msgHead := f5.WSProxyPackHead_S{}
	msgHead.PackLen = uint16(len(msgData))
	msgHead.MsgId = netMsg.GetNetMsgId()
	msgHead.MagicCode = f5.NET_MSG_MAGIC_CODE
	msgHead.SocketHandle = socketHandle

	buff := make([]byte, len(msgData)+f5.WSPROXYPACKHEAD_S_SIZE)
	msgHead.Write(buff, 0)
	copy(buff[f5.WSPROXYPACKHEAD_S_SIZE:], msgData[:])
	conn.Write(buff)
}

func (this* WSPListener) ProcessSSMMsg(handler *ss.SsNetMsgHandler, hdr *f5.MsgHdr) {
	ss.DispatchMsg(handler, hdr, this)
}
