package team

import (
	"q5"
	"f5"
)

type matchingInfo struct {
	team *team
	addTick int64
	entry q5.ListHead
	matchTimerWp *f5.TimerWp
	matchOk* matchingInfo
	matchTick int64
	lastHasOnlinePlayerTick int64
}

type matchMgr struct {
	teamUuidHash  map[string]*matchingInfo
	znGroupHash  map[int64]*q5.ListHead
}

func (this *matchMgr) Init() {
	this.teamUuidHash = make(map[string]*matchingInfo)
	this.znGroupHash = make(map[int64]*q5.ListHead)
}

func (this *matchMgr) UnInit() {
}

func (this *matchMgr) addMatch(team *team) {
	if this.getMatchingInfoByUuid(team.GetTeamUuid()) != nil {
		panic("111111")
		return
	}
	m := &matchingInfo{}
	m.team = team
	m.addTick = q5.GetTickCount()
	m.lastHasOnlinePlayerTick = q5.GetTickCount()
	m.entry.Init(m)
	m.matchTimerWp = f5.GetTimer().SetIntervalWp(1000, func (e int32, args* q5.Args) {
		if e == q5.TIMER_EXEC_EVENT {
			_matchMgr.execMatch(m)
		}
	})
	this.teamUuidHash[team.GetTeamUuid()] = m
	this.mustBeZn(team.GetZnKey()).AddTail(&m.entry)
}

func (this *matchMgr) cancelMatch(team *team) bool {
	m := this.getMatchingInfoByUuid(team.GetTeamUuid())
	if m != nil {
		if m.matchOk != nil {
			panic("cancelMatching error")
			return false
		}
		if !m.matchTimerWp.Expired() {
			f5.GetTimer().Delete(m.matchTimerWp)
		}
		m.entry.DelInit()
		delete(this.teamUuidHash, team.GetTeamUuid())
	}
	return false
}

func (this *matchMgr) getMatchingInfoByUuid(teamUuid string) *matchingInfo {
	team, ok := this.teamUuidHash[teamUuid]
	if ok {
		return team
	}
	return nil
}

func (this *matchMgr) mustBeZn(key int64) *q5.ListHead {
	val, ok := this.znGroupHash[key]
	if ok {
		return val
	} else {
		l := q5.NewListHead()
		this.znGroupHash[key] = l
		return l
	}
}

func (this *matchMgr) execMatch(m *matchingInfo) {
	if !m.team.NeedMatchRealPlayer() {
		this.oldExecMatch(m)
		return
	}
	if m.team.HasOnlinePlayer() {
		m.lastHasOnlinePlayerTick = q5.GetTickCount()
		l := this.mustBeZn(m.team.GetZnKey())
		l.ForEach(
			func (ele interface{}) bool {
				if targetM, ok := ele.(*matchingInfo); ok {
					if m != targetM && m.team.canMatch(targetM.team) {
						m.matchOk = targetM
						m.matchTick = q5.GetTickCount()

						targetM.matchOk = m
						targetM.matchTick = q5.GetTickCount()
						return false
					}
				}
				return true
			})
		if m.matchOk != nil {
			this.onMatchOk(m)
		}
	} else if q5.GetTickCount() - m.lastHasOnlinePlayerTick > 1000 * 60 * 30 {
		this.cancelMatch(m.team)
		_teamMgr.removeTeam(m.team.GetTeamUuid())
		f5.GetSysLog().Info("matchMgr.execMatch team Timeout:%s", m.team.GetTeamUuid())
	}
}

func (this *matchMgr) oldExecMatch(m *matchingInfo) {
	if q5.GetTickCount() - m.addTick > 1000 * 7 {
		m.entry.DelInit()
		f5.GetTimer().DeleteRunningTimer()
		m.team.setMatchOk(nil)
		startTime := f5.GetApp().GetNowSeconds()
		m.team.onMatchOk(startTime)
	} else {
		l := this.mustBeZn(m.team.GetZnKey())
		l.ForEach(
			func (ele interface{}) bool {
				if targetM, ok := ele.(*matchingInfo); ok {
					if m != targetM && m.team.canMatch(targetM.team) {
						m.matchOk = targetM
						m.matchTick = q5.GetTickCount()

						targetM.matchOk = m
						targetM.matchTick = q5.GetTickCount()
						return false
					}
				}
				return true
			})
		if m.matchOk != nil {
			this.onMatchOk(m)
		}
	}
}

func (this *matchMgr) onMatchOk(m *matchingInfo) {
	delete(this.teamUuidHash, m.team.GetTeamUuid())
	delete(this.teamUuidHash, m.matchOk.team.GetTeamUuid())
	m.entry.DelInit()
	m.matchOk.entry.DelInit()

	f5.GetTimer().DeleteRunningTimer()
	if !m.matchOk.matchTimerWp.Expired() {
		f5.GetTimer().Delete(m.matchOk.matchTimerWp)
	}

	m.team.setMatchOk(m.matchOk.team)
	m.matchOk.team.setMatchOk(m.team)

	startTime := f5.GetApp().GetNowSeconds()
	m.team.onMatchOk(startTime)
	m.matchOk.team.onMatchOk(startTime)
}
