package service

import (
	"crypto/sha1"
	"encoding/json"
	"f5"
	"fmt"
	"io"
	"main/mt"
	"q5"
	"time"
)

type mqConsumer struct {
}

func (this *mqConsumer) init() {
	subscribelist := map[string]string{
		"marquee": mt.Table.Config.GetNoticeUrl() + "/mqconsumer/webapp/marquee.php",
	}

	go this.SubscribeTopics(subscribelist)
}

func (this *mqConsumer) unInit() {
}

func (this *mqConsumer) RequestMarquee(content string, loop, interval int32) (rspstr string) {

	appkey := "YOUME1838B3633FF1410BDC9124BBD806F245B9D2E5AC"
	identifier := "admin"
	appsecret := "q6B570yTyj/00Nk4mYZtgDwyew5v05t13V1vo4mxpEuAaWUiinAyVxG41sNu3vsFe8sipOLfKfIVYGhzpQrqzvj5sId3mrBfj/s65a2gp36yDrI/nX5BnUAJB317SEosR6xLoPuhBvHU+/1DWI7nKSKaRNxnQiC46PJKFc2kX50BAAE="
	channelid := mt.Table.Config.GetNoticeChannelId()
	data := map[string]interface{}{
		"Notice": map[string]interface{}{
			"ChannelID":     channelid, //频道 ID，app 内唯一，字符串
			"NoticeType":    1,         //1 为跑马灯公告，2 为聊天框公告，3 为置顶公告，整数
			"LoopType":      1,         // 公告循环类型，1 为一次性公告，2 为周期性公告，整数
			"Title":         "",        // 公告标题，字符串
			"Content":       content,   // 公告内容，字符串
			"LinkKeyWords":  "",        // 链接关键字，字符串
			"LinkAddr":      "",        // 链接地址，字符串
			"Creator":       "",        // 公告添加人，字符串
			"EnableStatus":  2,         // 公告启用状态，1 为停用，2 为启用，整数
			"SendStartTime": "",        // 公告发送时间，格式为 HH:MM:SS，字符串
			"SendInstantly": 1,         // 是否即时发送，也属于公告 json 里的字段。1 是，0 否，不指定则不起作用
			"SendTimes":     loop,      // 公告发送次数，整数（需要传入）
			"SendInterval":  interval,  // 公告发送多次时，每次的间隔，单位秒，整数（需要传入）
		}}

	url := "https://sgapi.youme.im/v1/im/add_notice"
	if f5.IsTestEnv() {
		url = "https://api.youme.im/v1/im/add_notice"
	}
	curtime := q5.ToString(f5.GetApp().GetRealSeconds())
	sha1c := sha1.New()
	io.WriteString(sha1c, appsecret+curtime)
	checksum := fmt.Sprintf("%x", sha1c.Sum(nil))

	params := map[string]string{
		"appkey":     appkey,
		"identifier": identifier,
		"curtime":    curtime,
		"checksum":   checksum,
	}

	d, _ := json.Marshal(data)
	f5.GetHttpCliMgr().SendGoStyleJsonRspPost(
		url,
		params,
		nil,
		"application/json",
		string(d),
		func(rsp f5.HttpCliResponse) {
			if rsp.GetErr() != nil {
				rspstr = rsp.GetErr().Error()
				return
			}
			// f5.GetSysLog().Debug("marquee request rsp:%s", rsp.GetRawData())
		})
	return rspstr
}

func (this *mqConsumer) SubscribeTopics(topics map[string]string) {
	channel := fmt.Sprintf("%s%03x", f5.GetApp().GetPkgName(), f5.GetApp().GetInstanceId())
	suburl := mt.Table.Config.GetMqproxyUrl() + "/mqproxy/webapp/subscribe.php"
	for len(topics) > 0 {
		subscribedTopics := map[string]int{}
		for topicitem, cburl := range topics {
			f5.GetHttpCliMgr().SendGoStyleRequest(
				suburl,
				map[string]string{
					"topic":     topicitem,
					"channel":   channel,
					"noticeurl": cburl,
				},
				func(hcr f5.HttpCliResponse) {
					if hcr.GetErr() == nil && hcr.GetRawData() == "" {
						subscribedTopics[topicitem] = 0
					}
				})
		}
		for subitem := range subscribedTopics {
			delete(topics, subitem)
		}
		time.Sleep(time.Second * 10)
	}
}
