package listener

import (
	"encoding/json"
	"f5"
	"fmt"
	"main/service"
	"net"

	"jccommon"
	"main/mt"
	"q5"
	// . "main/global"
)

type UDPListener struct {
	udpconn *net.UDPConn
	ch      chan bool
}

func (this *UDPListener) Init() {
	this.ch = make(chan bool, 64)
	udpAddr, _ := net.ResolveUDPAddr("udp", "0.0.0.0:"+
		q5.ToString(mt.Table.MqproxyCluster.GetListenPort()))
	conn, err := net.ListenUDP("udp", udpAddr)
	if err != nil {
		panic(err)
	} else {
		this.udpconn = conn
		go this.serverRead()
	}
}

func (this *UDPListener) UnInit() {
	if this.udpconn != nil {
		this.udpconn.Close()
	}

	close(this.ch)
}

func (this *UDPListener) serverRead() {
	for {
		buf := make([]byte, jccommon.MAX_PACKET_LEN)
		bufLen, peer, err := this.udpconn.ReadFromUDP(buf)
		if err == nil && bufLen > 0 {
			this.ch <- true
			recvdata := buf[:bufLen]
			go func(addr *net.UDPAddr, data []byte, ch chan bool) {
				obj := struct {
					Req     string `json:"req"`
					Topic   string `json:"topic"`
					Channel string `json:"channel"`
					Notice  string `json:"noticeurl"`
					Expire  int32  `json:"expire"`
					Msg     string `json:"msg"`
				}{}
				if json.Unmarshal(data, &obj) == nil {
					retcode := 1
					content := ""
					switch obj.Req {
					case "pub":
						expire := obj.Expire
						if expire > 86400*7 {
							expire = 86400 * 7
						}
						if obj.Topic != "" && obj.Msg != "" {
							service.MqManager.PublishTopic(obj.Topic, obj.Msg, expire)
							retcode = 0
						}
					case "sub":
						if obj.Topic != "" && obj.Channel != "" {
							service.MqManager.SubscribeChannel(obj.Topic, obj.Channel, obj.Notice)
							retcode = 0
						}
					case "unsub":
						if obj.Topic != "" && obj.Channel != "" {
							service.MqManager.UnsubscribeChannel(obj.Topic, obj.Channel)
							retcode = 0
						}
					case "consume":
						if obj.Topic != "" && obj.Channel != "" {
							retcode = 0
							content = service.MqManager.ConsumeTopic(obj.Topic, obj.Channel)
						}
					}

					rsp := fmt.Sprintf("%d|%s", retcode, content)
					this.udpconn.WriteToUDP([]byte(rsp), addr)
				} else {
					f5.GetSysLog().Info("bad req:%s", string(data))
				}

				<-ch
			}(peer, recvdata, this.ch)
		}
	}
}
